@@ -1,15 +1,15 @@
 #include "controllers.hh"
 
+#include "chrono.hh"
 #include "fs.hh"
-#include <boost/algorithm/string.hpp>
-#include <boost/filesystem.hpp>
-#include <boost/lexical_cast.hpp>
-#include <boost/regex.hpp>
-#include <boost/smart_ptr/weak_ptr.hpp>
-#include <libxml++/libxml++.h>
-#include <SDL2/SDL_joystick.h>
+#include "libxml++-impl.hh"
+#include "unicode.hh"
+#include <regex>
+#include <SDL_joystick.h>
+
 #include <deque>
 #include <stdexcept>
+#include <algorithm>
 
 using namespace input;
 
@@ -28,7 +28,7 @@ namespace {
 		xmlpp::Attribute const* a = elem.get_attribute(attr);
 		if (!a) return false;
 		try {
-			var = boost::lexical_cast<T>(a->get_value());
+			var = sconv<T>(a->get_value());
 		} catch (std::exception&) {
 			throw XMLError(elem, "attribute " + attr + " value invalid: " + a->get_value());
 		}
@@ -40,31 +40,31 @@ namespace {
 	}
 	// Return a NavButton corresponding to an Event
 	NavButton navigation(Event const& ev) {
-		#define DEFINE_BUTTON(dt, btn, num, nav) if ((DEVTYPE_##dt == DEVTYPE_GENERIC || ev.devType == DEVTYPE_##dt) && ev.button == dt##_##btn) return nav;
+		#define DEFINE_BUTTON(dt, btn, num, nav) if ((DevType::dt == DevType::GENERIC || ev.devType == DevType::dt) && ev.button == ButtonId::dt##_##btn) return NavButton::nav;
 		#include "controllers-buttons.ii"
-		return NAV_NONE;
+		return NavButton::NONE;
 	}
 
 	std::ostream& operator<<(std::ostream& os, SourceId const& source) {
 		switch (source.type) {
-			case SOURCETYPE_NONE: return os << "(none)";
-			case SOURCETYPE_KEYBOARD: return os << "(keyboard " << source.device << " instrument " << source.channel << ")";
-			case SOURCETYPE_JOYSTICK: return os << "(joystick " << source.device << ")";
-			case SOURCETYPE_MIDI: return os << "(midi " << source.device << " channel " << source.channel << ")";
-			case SOURCETYPE_N: break;
+			case SourceType::NONE: return os << "(none)";
+			case SourceType::KEYBOARD: return os << "(keyboard " << source.device << " instrument " << source.channel << ")";
+			case SourceType::JOYSTICK: return os << "(joystick " << source.device << ")";
+			case SourceType::MIDI: return os << "(midi " << source.device << " channel " << source.channel << ")";
+			case SourceType::N: break;
 		}
 		throw std::logic_error("Unknown SOURCETYPE in controllers.cc SourceId operator<<");
 	}
 	std::string buttonDebug(DevType type, Button b) {
-		#define DEFINE_BUTTON(dt, btn, num, nav) if ((DEVTYPE_##dt == DEVTYPE_GENERIC || type == DEVTYPE_##dt) && b == dt##_##btn) \
-		  return #dt " " #btn " (" #nav ")";
+		#define DEFINE_BUTTON(dt, btn, num, nav) if ((DevType::dt == DevType::GENERIC || type == DevType::dt) && b == ButtonId::dt##_##btn) \
+		  return #dt " " #btn " (NavButton::" #nav ")";
 		#include "controllers-buttons.ii"
 		throw std::logic_error("Invalid Button value in controllers.cc buttonDebug");
 	}
 	std::ostream& operator<<(std::ostream& os, Event const& ev) {
 		os << ev.source << ' ';
 		// Print hw button info if the event is not assigned to a function, otherwise print assignments
-		if (ev.button == GENERIC_UNASSIGNED) {
+		if (ev.button == ButtonId::GENERIC_UNASSIGNED) {
 			if (hwIsAxis.matches(ev.hw)) {
 				os << "axis hw=" << ev.hw - hwIsAxis.min << " value=" << ev.value;
 			} else if (hwIsHat.matches(ev.hw)) {
@@ -89,7 +89,6 @@ struct ButtonMap {
 	Button map;  // Generic action
 	Button negative, positive;  // Half-axis movement
 	Button up, down, left, right;  // Hat direction
-	ButtonMap() { std::memset(this, 0, sizeof(*this)); }
 };
 
 typedef std::map<HWButton, ButtonMap> ButtonMapping;
@@ -101,7 +100,7 @@ struct ControllerDef {
 	SourceType sourceType;
 	DevType devType;
 	double latency;
-	boost::regex deviceRegex;
+	std::regex deviceRegex;
 	MinMax<unsigned> deviceMinMax;
 	MinMax<unsigned> channelMinMax;
 	ButtonMapping mapping;
@@ -110,7 +109,7 @@ struct ControllerDef {
 		if (ev.source.type != sourceType) return false;
 		if (!deviceMinMax.matches(ev.source.device)) return false;
 		if (!channelMinMax.matches(ev.source.channel)) return false;
-		if (!deviceRegex.empty() && !regex_search(devName, deviceRegex)) return false;
+		if (!regex_search(devName, deviceRegex)) return false;
 		return true;
 	}
 };
@@ -120,7 +119,7 @@ struct Controllers::Impl {
 	ControllerDefs m_controllerDefs;
 
 	typedef std::map<std::string, Button> NameToButton;
-	NameToButton m_buttons[DEVTYPE_N];
+	NameToButton m_buttons[to_underlying(DevType::N)];
 
 	typedef std::map<SourceId, ControllerDef const*> Assignments;
 	Assignments m_assignments;
@@ -130,22 +129,24 @@ struct Controllers::Impl {
 
 	std::deque<NavEvent> m_navEvents;
 	std::map<SourceId, DevicePtr> m_orphans;
-	std::map<SourceId, boost::weak_ptr<Device> > m_devices;
+	std::map<SourceId, std::weak_ptr<Device> > m_devices;
 	bool m_eventsEnabled;
 	
 	typedef std::pair<SourceId, ButtonId> UniqueButton;
 	std::map<UniqueButton, double> m_values;
 
 	std::map<NavButton, NavEvent> m_navRepeat;
-	
+
+	Time m_prevProcess{};
+
 	Impl(): m_eventsEnabled() {
-		#define DEFINE_BUTTON(devtype, button, num, nav) m_buttons[DEVTYPE_##devtype][#button] = devtype##_##button;
+		#define DEFINE_BUTTON(devtype, button, num, nav) m_buttons[to_underlying(DevType::devtype)][#button] = ButtonId::devtype##_##button;
 		#include "controllers-buttons.ii"
 		readControllers(getShareDir() / "config/controllers.xml");
 		readControllers(getConfigDir() / "controllers.xml");
-		m_hw[SOURCETYPE_KEYBOARD] = constructKeyboard();
-		m_hw[SOURCETYPE_JOYSTICK] = constructJoysticks();
-		if (Hardware::midiEnabled()) m_hw[SOURCETYPE_MIDI] = constructMidi();
+		m_hw[SourceType::KEYBOARD] = constructKeyboard();
+		m_hw[SourceType::JOYSTICK] = constructJoysticks();
+		if (Hardware::midiEnabled()) m_hw[SourceType::MIDI] = constructMidi();
 	}
 	
 	void readControllers(fs::path const& file) {
@@ -156,55 +157,54 @@ struct Controllers::Impl {
 		std::clog << "controllers/info: Parsing " << file << std::endl;
 		xmlpp::DomParser domParser(file.string());
 		try {
-			parseControllers(domParser, "/controllers/joystick/controller", SOURCETYPE_JOYSTICK);
-			parseControllers(domParser, "/controllers/midi/controller", SOURCETYPE_MIDI);
+			parseControllers(domParser, "/controllers/joystick/controller", SourceType::JOYSTICK);
+			parseControllers(domParser, "/controllers/midi/controller", SourceType::MIDI);
 		} catch (XMLError& e) {
 			int line = e.elem.get_line();
 			std::string name = e.elem.get_name();
-			throw std::runtime_error(file.string() + ":" + boost::lexical_cast<std::string>(line) + " element " + name + " " + e.message);
+			throw std::runtime_error(file.string() + ":" + std::to_string(line) + " element " + name + " " + e.message);
 		}
 	}
 
 	void parseControllers(xmlpp::DomParser const& dom, std::string const& path, SourceType sourceType) {
-		xmlpp::NodeSet n = dom.get_document()->get_root_node()->find(path);
-		for (xmlpp::NodeSet::const_iterator nodeit = n.begin(), end = n.end(); nodeit != end; ++nodeit) {
-			xmlpp::Element& elem = dynamic_cast<xmlpp::Element&>(**nodeit);
+		auto n = dom.get_document()->get_root_node()->find(path);
+		for (auto nodeit = n.begin(), end = n.end(); nodeit != end; ++nodeit) {
+			const xmlpp::Element& elem = dynamic_cast<const xmlpp::Element&>(**nodeit);
 			ControllerDef def;
 			def.name = getAttribute(elem, "name");
 			def.sourceType = sourceType;
 			// Device type
 			{
 				std::string type = getAttribute(elem, "type");
-				if (type == "guitar") def.devType = DEVTYPE_GUITAR;
-				else if (type == "drumkit") def.devType = DEVTYPE_DRUMS;
-				else if (type == "keytar") def.devType = DEVTYPE_KEYTAR;
-				else if (type == "piano") def.devType = DEVTYPE_PIANO;
-				else if (type == "dancepad") def.devType = DEVTYPE_DANCEPAD;
+				if (type == "guitar") def.devType = DevType::GUITAR;
+				else if (type == "drumkit") def.devType = DevType::DRUMS;
+				else if (type == "keytar") def.devType = DevType::KEYTAR;
+				else if (type == "piano") def.devType = DevType::PIANO;
+				else if (type == "dancepad") def.devType = DevType::DANCEPAD;
 				else {
 					std::clog << "controllers/warning: " << type << ": Unknown controller type in controllers.xml (skipped)" << std::endl;
 					continue;
 				}
 			}
-			xmlpp::NodeSet ns;
 			// Device description
-			ns = elem.find("description/text()");
-			if (ns.size() == 1) def.description = dynamic_cast<xmlpp::TextNode&>(*ns[0]).get_content();
+			auto ns = elem.find("description/text()");
+			if (ns.size() == 1) def.description = dynamic_cast<const xmlpp::TextNode&>(*ns[0]).get_content();
 			// Read filtering rules
 			ns = elem.find("device");
 			if (ns.size() == 1) {
-				xmlpp::Element& elem = dynamic_cast<xmlpp::Element&>(*ns[0]);
+				const xmlpp::Element& elem = dynamic_cast<const xmlpp::Element&>(*ns[0]);
 				std::string regex;
-				if (tryGetAttribute(elem, "regex", regex)) def.deviceRegex = regex;
+				if (tryGetAttribute(elem, "regex", regex)) { def.deviceRegex = regex; }
 				parse(def.deviceMinMax, elem);
 				double latency;
 				if (tryGetAttribute(elem, "latency", latency)) def.latency = latency;
 			}
 			ns = elem.find("channel");
-			if (ns.size() == 1) parse(def.channelMinMax, dynamic_cast<xmlpp::Element&>(*ns[0]));
+			if (ns.size() == 1) parse(def.channelMinMax, dynamic_cast<const xmlpp::Element&>(*ns[0]));
 			// Read button mapping
 			ns = elem.find("mapping/*");
-			for (xmlpp::NodeSet::const_iterator nodeit2 = ns.begin(), end = ns.end(); nodeit2 != end; ++nodeit2) {
-				xmlpp::Element& elem = dynamic_cast<xmlpp::Element&>(**nodeit2);
+			for (auto nodeit2 = ns.begin(), end = ns.end(); nodeit2 != end; ++nodeit2) {
+				const xmlpp::Element& elem = dynamic_cast<const xmlpp::Element&>(**nodeit2);
 				HWButton hw;
 				if (!tryGetAttribute(elem, "hw", hw)) throw XMLError(elem, "Mandatory attribute hw is missing or invalid");
 				// Axes and hats use different HWButton ranges internally
@@ -234,18 +234,18 @@ struct Controllers::Impl {
 	}
 	/// Find button by name, either of given type or of generic type
 	Button findButton(DevType type, std::string name) {
-		Button button = GENERIC_UNASSIGNED;
+		Button button = ButtonId::GENERIC_UNASSIGNED;
 		if (name.empty()) return button;
-		boost::algorithm::to_upper(name);
-		boost::algorithm::replace_all(name, "-", "_");
+		name = UnicodeUtil::toUpper(name);
+		std::replace( name.begin(), name.end(), '-', '_');
 		// Try getting button first from devtype-specific, then generic names
-		buttonByName(type, name, button) || buttonByName(DEVTYPE_GENERIC, name, button) ||
+		buttonByName(type, name, button) || buttonByName(DevType::GENERIC, name, button) ||
 		  std::clog << "controllers/warning: " << name << ": Unknown button name in controllers.xml." << std::endl;
 		return button;
 	}
 	/// Try to find button of specific type
 	bool buttonByName(DevType type, std::string const& name, Button& button) {
-		NameToButton const& n2b = m_buttons[type];
+		NameToButton const& n2b = m_buttons[to_underlying(type)];
 		auto it = n2b.find(name);
 		if (it == n2b.end()) return false;
 		button = it->second;
@@ -269,13 +269,9 @@ struct Controllers::Impl {
 	void enableEvents(bool state) {
 		m_eventsEnabled = state;
 		Hardware::enableKeyboardInstruments(state);
-		if (!state) {
-			m_orphans.clear();
-			m_devices.clear();
-		}
 	}
 	/// Do internal event processing (poll for MIDI events etc)
-	void process(boost::xtime const& now) {
+	void process(Time now) {
 		for (auto& typehw: m_hw) {
 			while (true) {
 				Event event;
@@ -284,20 +280,27 @@ struct Controllers::Impl {
 				pushHWEvent(event);
 			}
 		}
+		// Reset all key repeat timers if there is a latency spike
+		if (now - m_prevProcess > 50ms) {
+			for (auto& kv: m_navRepeat) kv.second.time = now;
+		}
+		m_prevProcess = now;
+		// Spawn key repeat events when holding buttons
 		for (auto& kv: m_navRepeat) {
 			NavEvent& ne = kv.second;
-			double delay = 2.0 / (10 + ne.repeat);
-			if (now - ne.time < delay) continue;  // Not yet time to repeat
+			Seconds delay(2.0 / (10 + ne.repeat));
+			Seconds since = now - ne.time;
+			if (since < delay) continue;  // Not yet time to repeat
 			// Emit auto-repeated event
 			// Note: We intentionally only emit one per frame (call to process) to avoid surprises when latency spikes occur.
 			++ne.repeat;
-			ne.time += delay;  // Increment rather than set to now, so that repeating is smoother.
-			std::clog << "controllers/debug: NavEvent auto repeat " << ne.repeat << " next=" << now - ne.time << " delay=" << delay << std::endl;
+			ne.time += clockDur(delay);  // Increment rather than set to now, so that repeating is smoother.
+			std::clog << "controllers/debug: NavEvent auto repeat " << ne.repeat << " after " << since.count() << " s, next delay " << delay.count() << "s " << std::endl;
 			m_navEvents.push_back(ne);
 		}
 	}
 	/// Handle an incoming SDL event
-	bool pushEvent(SDL_Event const& sdlEv, boost::xtime const& t) {
+	bool pushEvent(SDL_Event const& sdlEv, Time t) {
 		for (auto& typehw: m_hw) {
 			Event event;
 			event.time = t;
@@ -327,7 +330,7 @@ struct Controllers::Impl {
 			pushMappedEvent(event);  // This is for keyboard events mainly (they have no ControllerDefs)
 			return;
 		}
-		event.time += -def->latency;
+		event.time -= clockDur(def->latency * 1s);
 		event.devType = def->devType;
 		// Mapping from controllers.xml
 		auto it = def->mapping.find(event.hw);
@@ -352,33 +355,33 @@ struct Controllers::Impl {
 		}
 	}
 	bool pushMappedEvent(Event& ev) {
-		if (ev.button == GENERIC_UNASSIGNED) return false;
+		if (ev.button == ButtonId::GENERIC_UNASSIGNED) return false;
 		if (!valueChanged(ev)) return false;  // Avoid repeated or other useless events
 		std::clog << "controllers/debug: processing " << ev << std::endl;
 		ev.nav = navigation(ev);
 		// Emit nav event (except if device is currently registered for events)
-		if (ev.nav != NAV_NONE) {
+		if (ev.nav != NavButton::NONE) {
 			NavEvent ne(ev);
 			// Menu navigation mapping
 			{
-				bool vertical = (ev.devType == DEVTYPE_GUITAR);
-				if (ne.button == NAV_UP) ne.menu = (vertical ? NAVMENU_A_PREV : NAVMENU_B_PREV);
-				else if (ne.button == NAV_DOWN) ne.menu = (vertical ? NAVMENU_A_NEXT : NAVMENU_B_NEXT);
-				else if (ne.button == NAV_LEFT) ne.menu = (vertical ? NAVMENU_B_PREV : NAVMENU_A_PREV);
-				else if (ne.button == NAV_RIGHT) ne.menu = (vertical ? NAVMENU_B_NEXT : NAVMENU_A_NEXT);
+				bool vertical = (ev.devType == DevType::GUITAR);
+				if (ne.button == NavButton::UP) ne.menu = (vertical ? NavMenu::A_PREV : NavMenu::B_PREV);
+				else if (ne.button == NavButton::DOWN) ne.menu = (vertical ? NavMenu::A_NEXT : NavMenu::B_NEXT);
+				else if (ne.button == NavButton::LEFT) ne.menu = (vertical ? NavMenu::B_PREV : NavMenu::A_PREV);
+				else if (ne.button == NavButton::RIGHT) ne.menu = (vertical ? NavMenu::B_NEXT : NavMenu::A_NEXT);
 			}
 			if (ev.value != 0.0) {
 				m_navEvents.push_back(ne);
-				if (ne.button >= NAV_REPEAT) m_navRepeat.insert(std::make_pair(ne.button, ne));
+				if (ne.button >= NavButton::REPEAT) m_navRepeat.insert(std::make_pair(ne.button, ne));
 			} else {
-				if (ne.button >= NAV_REPEAT) m_navRepeat.erase(ne.button);
+				if (ne.button >= NavButton::REPEAT) m_navRepeat.erase(ne.button);
 			}
 		}
 		if (m_eventsEnabled) {
 			// Emit Event and construct a new Device first if needed
 			DevicePtr ptr = m_devices[ev.source].lock();
 			if (!ptr) {
-				ptr.reset(new Device(ev.source, ev.devType));
+				ptr = std::make_shared<Device>(ev.source, ev.devType);
 				m_orphans[ev.source] = ptr;
 				m_devices[ev.source] = ptr;
 			}
@@ -404,8 +407,8 @@ Controllers::~Controllers() {}
 bool Controllers::getNav(NavEvent& ev) { return self->getNav(ev); }
 DevicePtr Controllers::registerDevice(SourceId const& source) { return self->registerDevice(source); }
 void Controllers::enableEvents(bool state) { self->enableEvents(state); }
-void Controllers::process(boost::xtime const& now) { self->process(now); }
-bool Controllers::pushEvent(SDL_Event const& ev, boost::xtime const& t) { return self->pushEvent(ev, t); }
+void Controllers::process(Time now) { self->process(now); }
+bool Controllers::pushEvent(SDL_Event const& ev, Time t) { return self->pushEvent(ev, t); }
 
 bool Device::getEvent(Event& ev) {
 	if (m_events.empty()) return false;
