@@ -2,7 +2,6 @@
 #include "song.hh"
 #include "i18n.hh"
 
-#include <boost/lexical_cast.hpp>
 #include <stdexcept>
 #include <algorithm>
 
@@ -32,7 +31,7 @@ namespace {
 	int getNextBigStreak(int prev) { return prev + 10; }
 
 	/// Get an accuracy value [0, 1] for the error offset (in seconds)
-	double accuracy(double error) { return 1.0 - (std::abs(error) / maxTolerance); };
+	double accuracy(double error) { return 1.0 - (std::abs(error) / maxTolerance); }
 
 	/// Gives points based on error from a perfect hit
 	double points(double error) {
@@ -49,21 +48,21 @@ namespace {
 	std::string getRank(double error) {
 		double ac = accuracy(error);
 		if (error < 0.0) {
-			if (ac > .90) return "Perfect!";
-			if (ac > .80) return "Excellent!-";
-			if (ac > .70) return "Great!-";
-			if (ac > .60) return " Good!- ";
-			if (ac > .40) return "  OK!-  ";
-			if (ac > .20) return "Late!-";
-			return "Way off!";
+			if (ac > .90) return _("Perfect!");
+			if (ac > .80) return _("Excellent!-");
+			if (ac > .70) return _("Great!-");
+			if (ac > .60) return _(" Good!- ");
+			if (ac > .40) return _("  OK!-  ");
+			if (ac > .20) return _("Late!-");
+			return _("Way off!");
 		} else {
-			if (ac > .90) return "Perfect!";
-			if (ac > .80) return "-Excellent!";
-			if (ac > .70) return "-Great!";
-			if (ac > .60) return " -Good! ";
-			if (ac > .40) return "  -OK!  ";
-			if (ac > .20) return "-Early!";
-			return "Way off!";
+			if (ac > .90) return _("Perfect!");
+			if (ac > .80) return _("-Excellent!");
+			if (ac > .70) return _("-Great!");
+			if (ac > .60) return _(" -Good! ");
+			if (ac > .40) return _("  -OK!  ");
+			if (ac > .20) return _("-Early!");
+			return _("Way off!");
 		}
 	}
 
@@ -78,7 +77,7 @@ namespace {
 /// Constructor
 DanceGraph::DanceGraph(Audio& audio, Song const& song, input::DevicePtr dev):
   InstrumentGraph(audio, song, dev),
-  m_level(BEGINNER),
+  m_level(DanceDifficulty::BEGINNER),
   m_beat(findFile("dancebeat.svg")),
   m_arrows(findFile("arrows.svg")),
   m_arrows_cursor(findFile("arrows_cursor.svg")),
@@ -122,9 +121,9 @@ void DanceGraph::setupJoinMenu() {
 		ConfigItem::OptionList ol;
 		int i = 0, cur = 0;
 		// Add difficulties to the option list
-		for (int level = 0; level < DIFFICULTYCOUNT; ++level) {
+		for (int level = 0; level < to_underlying(DanceDifficulty::COUNT); ++level) {
 			if (difficulty(DanceDifficulty(level), true)) {
-				ol.push_back(boost::lexical_cast<std::string>(level));
+				ol.push_back(std::to_string(level));
 				if (DanceDifficulty(level) == m_level) cur = i;
 				++i;
 			}
@@ -196,18 +195,18 @@ std::string DanceGraph::getTrack() const {
 
 /// Get the difficulty as displayable string
 std::string DanceGraph::getDifficultyString() const {
-	return _(diffv[m_level].c_str());
+	return _(diffv[to_underlying(m_level)].c_str());
 }
 
 /// Get a string id for track and difficulty
 std::string DanceGraph::getModeId() const {
-	return m_gamingMode + " - " + diffv[m_level] + (isKeyboard() ? " (kbd)" : "");
+	return m_gamingMode + " - " + diffv[to_underlying(m_level)] + (isKeyboard() ? " (kbd)" : "");
 }
 
 /// Attempt to change the difficulty by a step
 void DanceGraph::changeDifficulty(int delta) {
-	int newLevel = m_level + delta;
-	if(newLevel >= DIFFICULTYCOUNT || newLevel < 0) return; // Out of bounds
+	int newLevel = to_underlying(m_level) + delta;
+	if(newLevel >= to_underlying(DanceDifficulty::COUNT) || newLevel < 0) return; // Out of bounds
 	auto it = m_song.danceTracks.find(m_gamingMode);
 	if(it->second.find((DanceDifficulty)newLevel) != it->second.end())
 		difficulty((DanceDifficulty)newLevel);
@@ -260,33 +259,34 @@ void DanceGraph::engine() {
 		m_dead = 0; // Keep alive
 		// Menu keys
 		if (menuOpen() && ev.value != 0.0) {
-			if (ev.nav == input::NAV_START || ev.nav == input::NAV_CANCEL) m_menu.close();
-			else if (ev.nav == input::NAV_RIGHT) m_menu.action(1);
-			else if (ev.nav == input::NAV_LEFT) m_menu.action(-1);
-			else if (ev.nav == input::NAV_UP) m_menu.move(-1);
-			else if (ev.nav == input::NAV_DOWN) m_menu.move(1);
+			if (ev.nav == input::NavButton::START || ev.nav == input::NavButton::CANCEL) m_menu.close();
+			else if (ev.nav == input::NavButton::RIGHT) m_menu.action(1);
+			else if (ev.nav == input::NavButton::LEFT) m_menu.action(-1);
+			else if (ev.nav == input::NavButton::UP) m_menu.move(-1);
+			else if (ev.nav == input::NavButton::DOWN) m_menu.move(1);
 			difficulty_changed = true;
 			// See if anything changed
 			if (m_selectedTrack.so() != m_gamingMode) setTrack(m_selectedTrack.so());
-			else if (boost::lexical_cast<int>(m_selectedDifficulty.so()) != m_level)
-				difficulty(DanceDifficulty(boost::lexical_cast<int>(m_selectedDifficulty.so())));
+			else if (std::stoi(m_selectedDifficulty.so()) != to_underlying(m_level))
+				difficulty(DanceDifficulty(std::stoi(m_selectedDifficulty.so())));
 			else if (m_rejoin.b()) { unjoin(); setupJoinMenu(); m_dev->pushEvent(input::Event()); /* FIXME: HACK? */ }
 			// Sync dynamic stuff
 			updateJoinMenu();
 		// Open Menu
 		} else if (!menuOpen() && ev.value != 0.0) {
-			if (ev.nav == input::NAV_CANCEL || ev.nav == input::NAV_START) m_menu.open();
+			if (ev.nav == input::NavButton::CANCEL || ev.nav == input::NavButton::START) m_menu.open();
 		}
-		if (ev.button < max_panels) {
+		auto buttonId = to_underlying(ev.button.id);
+		if (buttonId < static_cast<decltype(buttonId)>(max_panels)) {
 			// Gaming controls
 			if (ev.value == 0.0) {
-				m_pressed[ev.button] = false;
+				m_pressed[buttonId] = false;
 				dance(time, ev);
-				m_pressed_anim[ev.button].setTarget(0.0);
+				m_pressed_anim[buttonId].setTarget(0.0);
 			} else if (ev.value != 0.0) {
-				m_pressed[ev.button] = true;
+				m_pressed[buttonId] = true;
 				dance(time, ev);
-				m_pressed_anim[ev.button].setValue(1.0);
+				m_pressed_anim[buttonId].setValue(1.0);
 			}
 		}
 	}
@@ -297,9 +297,9 @@ void DanceGraph::engine() {
 	// Notes gone by
 	for (DanceNotes::iterator& it = m_notesIt; it != m_notes.end() && time > it->note.end + maxTolerance; it++) {
 		if(!it->isHit) { // Missed
-			if (it->note.type != Note::MINE) m_streak = 0;
+			if (it->note.type != Note::Type::MINE) m_streak = 0;
 		} else { // Hit, add score
-			if(it->note.type != Note::MINE) m_score += it->score;
+			if(it->note.type != Note::Type::MINE) m_score += it->score;
 			if(!it->releaseTime) it->releaseTime = time;
 		}
 		if (!joining(time)) ++m_dead;  // Increment dead counter (but not while joining)
@@ -308,7 +308,7 @@ void DanceGraph::engine() {
 
 	// Holding button when mine comes?
 	for (auto it = m_notesIt; it != m_notes.end() && time <= it->note.begin + maxTolerance; ++it) {
-		if(!it->isHit && it->note.type == Note::MINE && m_pressed[it->note.note] &&
+		if(!it->isHit && it->note.type == Note::Type::MINE && m_pressed[it->note.note] &&
 		  it->note.begin >= time - maxTolerance && it->note.end <= time + maxTolerance) {
 			it->isHit = true;
 			m_score -= points(0);
@@ -318,7 +318,7 @@ void DanceGraph::engine() {
 	// Check if a long streak goal has been reached
 	if (m_streak >= getNextBigStreak(m_bigStreak)) {
 		m_bigStreak = getNextBigStreak(m_bigStreak);
-		m_popups.push_back(Popup(boost::lexical_cast<std::string>(unsigned(m_bigStreak)) + "\n" + _("Streak!"),
+		m_popups.push_back(Popup(std::to_string(unsigned(m_bigStreak)) + "\n" + _("Streak!"),
 		  Color(1.0, 0.0, 0.0), 1.0, m_popupText.get()));
 	}
 }
@@ -327,7 +327,7 @@ void DanceGraph::engine() {
 void DanceGraph::dance(double time, input::Event const& ev) {
 	// Handle release events
 	if (ev.value == 0.0) {
-		auto it = m_activeNotes[ev.button];
+		auto it = m_activeNotes[to_underlying(ev.button.id)];
 		if(it != m_notes.end()) {
 			if(!it->releaseTime && it->note.end > time + maxTolerance) {
 				it->releaseTime = time;
@@ -338,11 +338,12 @@ void DanceGraph::dance(double time, input::Event const& ev) {
 		return;
 	}
 
+	auto buttonId = to_underlying(ev.button.id);
 	// So it was a PRESS event
 	for (auto it = m_notesIt; it != m_notes.end() && time <= it->note.end + maxTolerance; ++it) {
-		if(!it->isHit && std::abs(time - it->note.begin) <= maxTolerance && ev.button == unsigned(it->note.note)) {
+		if(!it->isHit && std::abs(time - it->note.begin) <= maxTolerance && buttonId == static_cast<decltype(buttonId)>(it->note.note)) {
 			it->isHit = true;
-			if (it->note.type != Note::MINE) {
+			if (it->note.type != Note::Type::MINE) {
 				it->score = points(it->note.begin - time);
 				it->error = it->note.begin - time;
 				m_streak++;
@@ -351,7 +352,7 @@ void DanceGraph::dance(double time, input::Event const& ev) {
 				m_score -= points(0);
 				m_streak = 0;
 			}
-			m_activeNotes[ev.button] = it;
+			m_activeNotes[buttonId] = it;
 			break;
 		}
 	}
@@ -409,13 +410,14 @@ void DanceGraph::draw(double time) {
 		// Arrows on cursor
 		{
 			UseShader us(getShader("dancenote"));
-			us()["clock"].set(float(time));
-			us()["noteType"].set(0);
-			us()["scale"].set(getScale());
+			m_uniforms.clock = static_cast<float>(time);
+			m_uniforms.noteType = 0;
+			m_uniforms.scale = getScale();
 			for (unsigned arrow_i = 0; arrow_i < m_pads; ++arrow_i) {
 				float l = m_pressed_anim[arrow_i].get();
-				us()["hitAnim"].set(l);
-				us()["position"].set(panel2x(arrow_i), time2y(0.0));
+				m_uniforms.hitAnim = l;
+				m_uniforms.position = glmath::vec2(panel2x(arrow_i), time2y(0.0));
+				glBufferSubData(GL_UNIFORM_BUFFER, m_uniforms.offset(), m_uniforms.size(), &m_uniforms);
 				drawArrow(arrow_i, m_arrows_cursor);
 			}
 		}
@@ -458,7 +460,7 @@ void DanceGraph::drawNote(DanceNote& note, double time) {
 	float tBeg = note.note.begin - time;
 	float tEnd = note.note.end - time;
 	int arrow_i = note.note.note;
-	bool mine = note.note.type == Note::MINE;
+	bool mine = note.note.type == Note::Type::MINE;
 	float x = panel2x(arrow_i);
 	float yBeg = time2y(tBeg);
 	float yEnd = time2y(tEnd);
@@ -472,9 +474,9 @@ void DanceGraph::drawNote(DanceNote& note, double time) {
 
 	{
 		UseShader us(getShader("dancenote"));
-		us()["hitAnim"].set(float(glow));
-		us()["clock"].set(float(time));
-		us()["scale"].set(getScale());
+		m_uniforms.hitAnim = static_cast<float>(glow);
+		m_uniforms.clock = static_cast<float>(time);
+		m_uniforms.scale = getScale();
 
 		if (yEnd - yBeg > arrowSize) {
 			// Draw holds
@@ -483,8 +485,9 @@ void DanceGraph::drawNote(DanceNote& note, double time) {
 				yEnd = std::max(time2y(0.0), yEnd);
 			}
 			if (note.releaseTime > 0) yBeg = time2y(note.releaseTime - time); // Oh noes, it got released!
-			us()["noteType"].set(2);
-			us()["position"].set(x, yBeg);
+			m_uniforms.noteType = 2;
+			m_uniforms.position = glmath::vec2(x, yBeg);
+			glBufferSubData(GL_UNIFORM_BUFFER, m_uniforms.offset(), m_uniforms.size(), &m_uniforms);
 			// Draw begin
 			drawArrow(arrow_i, m_arrows_hold, 0.0f, 1.0f/3.0f);
 			if (yEnd - yBeg > 0) {
@@ -503,8 +506,9 @@ void DanceGraph::drawNote(DanceNote& note, double time) {
 		} else {
 			// Draw short note
 			if (mine && note.isHit) yBeg = time2y(0.0);
-			us()["noteType"].set(mine ? 3 : 1);
-			us()["position"].set(x, yBeg);
+			m_uniforms.noteType = (mine ? 3 : 1);
+			m_uniforms.position = glmath::vec2(x, yBeg);
+			glBufferSubData(GL_UNIFORM_BUFFER, m_uniforms.offset(), m_uniforms.size(), &m_uniforms);
 			drawArrow((mine ? -1 : arrow_i), (mine ? m_mine : m_arrows));
 		}
 	}
@@ -523,7 +527,7 @@ void DanceGraph::drawNote(DanceNote& note, double time) {
 			double sc = getScale() * 0.6 * arrowSize * (3.0 + glow);
 			Transform trans(glmath::translate(glmath::vec3(0.0, 0.0, 0.5 * glow))); // Slightly elevated
 			ColorTrans c(Color::alpha(std::sqrt(alpha)));
-			m_popupText->render(text);
+			m_popupText->render(_(text));
 			m_popupText->dimensions().middle(x).center(time2y(0.0)).stretch(sc, sc/2.0);
 			m_popupText->draw();
 		}
@@ -535,10 +539,10 @@ void DanceGraph::drawInfo(double /*time*/, Dimensions dimensions) {
 	if (!menuOpen()) {
 		// Draw scores
 		m_text.dimensions.screenBottom(-0.35).middle(0.32 * dimensions.w());
-		m_text.draw(boost::lexical_cast<std::string>(unsigned(getScore())));
+		m_text.draw(std::to_string(unsigned(getScore())));
 		m_text.dimensions.screenBottom(-0.32).middle(0.32 * dimensions.w());
-		m_text.draw(boost::lexical_cast<std::string>(unsigned(m_streak)) + "/"
-		  + boost::lexical_cast<std::string>(unsigned(m_longestStreak)));
+		m_text.draw(std::to_string(unsigned(m_streak)) + "/"
+		  + std::to_string(unsigned(m_longestStreak)));
 	}
 	drawPopups();
 }
