@@ -42,12 +42,12 @@ InstrumentGraph::InstrumentGraph(Audio& audio, Song const& song, input::DevicePt
 	double time = m_audio.getPosition();
 	m_jointime = time < 0.0 ? -1.0 : time + join_delay;
 
-	m_popupText.reset(new SvgTxtThemeSimple(findFile("sing_popup_text.svg"), config["graphic/text_lod"].f()));
-	m_menuTheme.reset(new ThemeInstrumentMenu());
+	m_popupText = std::make_unique<SvgTxtThemeSimple>(findFile("sing_popup_text.svg"), config["graphic/text_lod"].f());
+	m_menuTheme = std::make_unique<ThemeInstrumentMenu>();
 	for (auto& elem: m_pressed) elem = false;
 }
 
-InstrumentGraph::~InstrumentGraph() {}  // For destruction of scoped_ptrs (only forward-declared in header)
+InstrumentGraph::~InstrumentGraph() = default;  // For destruction of unique_ptrs (only forward-declared in header)
 
 bool InstrumentGraph::dead() const { return m_jointime != m_jointime || m_dead >= death_delay; }
 
@@ -82,7 +82,7 @@ void InstrumentGraph::drawMenu() {
 	ViewTrans view(m_cx.get(), 0.0, 0.75);  // Apply a per-player local perspective
 	if (m_menu.empty()) return;
 	Dimensions dimensions(1.0); // FIXME: bogus aspect ratio (is this fixable?)
-	if (getGraphType() == input::DEVTYPE_DANCEPAD) dimensions.screenTop().middle().stretch(m_width.get(), 1.0);
+	if (getGraphType() == input::DevType::DANCEPAD) dimensions.screenTop().middle().stretch(m_width.get(), 1.0);
 	else dimensions.screenBottom().middle().fixedWidth(std::min(m_width.get(), 0.5));
 	ThemeInstrumentMenu& th = *m_menuTheme;
 	th.back_h.dimensions.fixedHeight(0.08f);
@@ -98,7 +98,7 @@ void InstrumentGraph::drawMenu() {
 	// All these vars are ultimately affected by the scaling matrix
 	const float txth = th.option_selected.h();
 	const float button_margin = m_arrow_up.dimensions.w()
-		* (isKeyboard() && getGraphType() != input::DEVTYPE_DANCEPAD ? 2.0f : 1.0f);
+		* (isKeyboard() && getGraphType() != input::DevType::DANCEPAD ? 2.0f : 1.0f);
 	const float step = txth * 0.7f;
 	const float h = m_menu.getOptions().size() * step + step;
 	float y = -h * .5f + step;
@@ -120,7 +120,7 @@ void InstrumentGraph::drawMenu() {
 		// Selected item
 		} else {
 			// Left/right Icons
-			if (getGraphType() == input::DEVTYPE_DRUMS) {
+			if (getGraphType() == input::DevType::DRUMS) {
 				// Drum colors are mirrored
 				m_arrow_left.dimensions.middle(xx + button_margin).center(y);
 				m_arrow_right.dimensions.middle(x - button_margin).center(y);
@@ -132,7 +132,7 @@ void InstrumentGraph::drawMenu() {
 			m_arrow_right.draw();
 
 			// Up/down icons
-			if (getGraphType() != input::DEVTYPE_GUITAR) {
+			if (getGraphType() != input::DevType::GUITAR) {
 				if (i > 0) { // Up
 					m_arrow_up.dimensions.middle(x - button_margin).center(y - step);
 					m_arrow_up.draw();
@@ -144,22 +144,22 @@ void InstrumentGraph::drawMenu() {
 			}
 
 			// Draw the key letters for keyboard (not for dancepad)
-			if (isKeyboard() && getGraphType() != input::DEVTYPE_DANCEPAD) {
+			if (isKeyboard() && getGraphType() != input::DevType::DANCEPAD) {
 				float leftx = x - button_margin*0.75f;
 				float rightx = xx + button_margin*0.25f;
 				{
-					std::string hintletter = (getGraphType() == input::DEVTYPE_GUITAR ? (m_leftymode.b() ? "Z" : "1") : "U");
+					std::string hintletter = (getGraphType() == input::DevType::GUITAR ? (m_leftymode.b() ? "Z" : "1") : "U");
 					SvgTxtTheme& hintfont = th.getCachedOption(hintletter);
 					hintfont.dimensions.left(leftx).center(y);
 					hintfont.draw(hintletter);
 				}{
-					std::string hintletter = (getGraphType() == input::DEVTYPE_GUITAR ? (m_leftymode.b() ? "X" : "2") : "P");
+					std::string hintletter = (getGraphType() == input::DevType::GUITAR ? (m_leftymode.b() ? "X" : "2") : "P");
 					SvgTxtTheme& hintfont = th.getCachedOption(hintletter);
 					hintfont.dimensions.right(rightx).center(y);
 					hintfont.draw(hintletter);
 				}
 				// Only drums has up/down
-				if (getGraphType() == input::DEVTYPE_DRUMS) {
+				if (getGraphType() == input::DevType::DRUMS) {
 					if (i > 0) {  // Up
 						SvgTxtTheme& hintfont = th.getCachedOption("I");
 						hintfont.dimensions.left(leftx).center(y - step);
@@ -203,7 +203,7 @@ void InstrumentGraph::drawPopups() {
 void InstrumentGraph::handleCountdown(double time, double beginTime) {
 	if (!dead() && time < beginTime && time >= beginTime - m_countdown - 1) {
 		m_popups.push_back(Popup(m_countdown > 0 ?
-		  std::string("- ") +boost::lexical_cast<std::string>(unsigned(m_countdown))+" -" : "Rock On!",
+		  std::string("- ") +std::to_string(unsigned(m_countdown))+" -" : "Rock On!",
 		  Color(0.0, 0.0, 1.0), 2.0, m_popupText.get()));
 		  --m_countdown;
 	}
@@ -219,7 +219,7 @@ Color const& InstrumentGraph::color(unsigned fret) const {
 		Color(0.9, 0.4, 0.0)
 	};
 	if (fret >= m_pads) throw std::logic_error("Invalid fret number in InstrumentGraph::color");
-	if (getGraphType() == input::DEVTYPE_DRUMS) {
+	if (getGraphType() == input::DevType::DRUMS) {
 		if (fret == 0) fret = 4;
 		else if (fret == 4) fret = 0;
 	}
