@@ -1,18 +1,20 @@
-
 #include "backgrounds.hh"
+#include "chrono.hh"
 #include "config.hh"
 #include "controllers.hh"
 #include "database.hh"
+#include "engine.hh"
 #include "fs.hh"
 #include "glutil.hh"
 #include "i18n.hh"
 #include "log.hh"
+#include "platform.hh"
 #include "profiler.hh"
 #include "screen.hh"
 #include "songs.hh"
 #include "video_driver.hh"
 #include "webcam.hh"
-#include "xtime.hh"
+#include "webserver.hh"
 
 // Screens
 #include "screen_intro.hh"
@@ -24,26 +26,24 @@
 #include "screen_players.hh"
 #include "screen_playlist.hh"
 
-#include <boost/bind.hpp>
-#include <boost/format.hpp>
 #include <boost/program_options.hpp>
-#include <boost/thread.hpp>
+#include <cstdlib>
 #include <csignal>
 #include <string>
+#include <thread>
 #include <vector>
-#include <cstdlib>
 
 // Disable main level exception handling for debug builds (because gdb cannot properly catch throwing otherwise)
 #ifdef NDEBUG
 #define RUNTIME_ERROR std::runtime_error
 #define EXCEPTION std::exception
 #else
-namespace { struct Nothing { char const* what() const { return NULL; } }; }
+namespace { struct Nothing { char const* what() const { return nullptr; } }; }
 #define RUNTIME_ERROR Nothing
 #define EXCEPTION Nothing
 #endif
 
-volatile bool g_quit = false;
+std::atomic<bool> g_quit{ false };
 
 bool g_take_screenshot = false;
 
@@ -66,43 +66,36 @@ static void signalSetup() {
 /// can be thrown as an exception to quit the game
 struct QuitNow {};
 
-static void checkEvents(Game& gm) {
+static void checkEvents(Game& gm, Time eventTime) {
 	if (g_quit) {
 		std::cerr << "Terminating, please wait... (or kill the process)" << std::endl;
 		throw QuitNow();
 	}
+	Window& window = gm.window();
 	SDL_Event event;
-	while(SDL_PollEvent(&event) == 1) {
+	while (SDL_PollEvent(&event) == 1) {
 		// Let the navigation system grab any and all SDL events
-		boost::xtime eventTime = now();
 		gm.controllers.pushEvent(event, eventTime);
-		switch(event.type) {
-		  case SDL_QUIT:
-			gm.finished();
-			break;
-		  case SDL_KEYDOWN: {
-			int keypressed  = event.key.keysym.scancode;
-			uint16_t modifier = event.key.keysym.mod;
-			if (((keypressed == SDL_SCANCODE_RETURN || keypressed == SDL_SCANCODE_KP_ENTER) && modifier & KMOD_ALT) || keypressed == SDL_SCANCODE_F11) {
+		auto type = event.type;
+		if (type == SDL_WINDOWEVENT) window.event(event.window.event, event.window.data1, event.window.data2);
+		if (type == SDL_QUIT) gm.finished();
+		if (type == SDL_KEYDOWN) {
+			auto key  = event.key.keysym.scancode;
+			auto mod = event.key.keysym.mod;
+			bool altEnter = (key == SDL_SCANCODE_RETURN || key == SDL_SCANCODE_KP_ENTER) && mod & KMOD_ALT;  // Alt+Enter
+			bool modF = key == SDL_SCANCODE_F && mod & KMOD_CTRL && mod & KMOD_GUI;  // MacOS Ctrl+Cmd+F
+			if (altEnter || modF || key == SDL_SCANCODE_F11) {
 				config["graphic/fullscreen"].b() = !config["graphic/fullscreen"].b();
 				continue; // Already handled here...
 			}
-			if (keypressed == SDL_SCANCODE_PRINTSCREEN || (keypressed == SDL_SCANCODE_F12 && (modifier & KMOD_CTRL))) {
+			if (key == SDL_SCANCODE_PRINTSCREEN || (key == SDL_SCANCODE_F12 && (mod & Platform::shortcutModifier()))) {
 				g_take_screenshot = true;
 				continue; // Already handled here...
 			}
-			if (keypressed == SDL_SCANCODE_F4 && modifier & KMOD_ALT) {
+			if (key == SDL_SCANCODE_F4 && mod & KMOD_ALT) {
 				gm.finished();
 				continue; // Already handled here...
 			}
-			break;
-		  }
-		case SDL_WINDOWEVENT:
-			switch (event.window.event) {
-			  case SDL_WINDOWEVENT_RESIZED:
-				gm.window().resize(event.window.data1, event.window.data2);
-				break;
-			}
 		}
 		// Screens always receive SDL events that were not already handled here
 		gm.getCurrentScreen()->manageEvent(event);
@@ -110,42 +103,46 @@ static void checkEvents(Game& gm) {
 	for (input::NavEvent event; gm.controllers.getNav(event); ) {
 		input::NavButton nav = event.button;
 		// Volume control
-		if (nav == input::NAV_VOLUME_UP || nav == input::NAV_VOLUME_DOWN) {
+		if (nav == input::NavButton::VOLUME_UP || nav == input::NavButton::VOLUME_DOWN) {
 			std::string curS = gm.getCurrentScreen()->getName();
 			// Pick proper setting
 			std::string which_vol = (curS == "Sing" || curS == "Practice")
 			  ? "audio/music_volume" : "audio/preview_volume";
 			// Adjust value
-			if (nav == input::NAV_VOLUME_UP) ++config[which_vol]; else --config[which_vol];
+			if (nav == input::NavButton::VOLUME_UP) ++config[which_vol]; else --config[which_vol];
 			// Show message
 			gm.flashMessage(config[which_vol].getShortDesc() + ": " + config[which_vol].getValue());
 			continue; // Already handled here...
 		}
 		// If a dialog is open, any nav event will close it
-		if (gm.isDialogOpen()) { gm.closeDialog(); continue; }
+		if (gm.isDialogOpen()) { gm.closeDialog(); }
 		// Let the current screen handle other events
 		gm.getCurrentScreen()->manageEvent(event);
 	}
 
-	// Need to toggle full screen mode?
-	if (config["graphic/fullscreen"].b() != gm.window().getFullscreen()) {
-		gm.window().setFullscreen(config["graphic/fullscreen"].b());
-		gm.reloadGL();
-	}
+	// Need to toggle full screen mode or adjust resolution?
+	window.resize();
 }
 
 void mainLoop(std::string const& songlist) {
+	Platform platform;
 	std::clog << "core/notice: Starting the audio subsystem (errors printed on console may be ignored)." << std::endl;
 	Audio audio;
 	std::clog << "core/info: Loading assets." << std::endl;
-	Gettext localization(PACKAGE);
-	Window window(config["graphic/window_width"].i(), config["graphic/window_height"].i(), config["graphic/fullscreen"].b());
-	SurfaceLoader m_loader;
+	TranslationEngine localization(PACKAGE);
+	std::unique_ptr<Window> window;
+	TextureLoader m_loader;
 	Backgrounds backgrounds;
 	Database database(getConfigDir() / "database.xml");
 	Songs songs(database, songlist);
 	loadFonts();
-	Game gm(window);
+	try {
+		window = std::make_unique<Window>();
+		} catch (RUNTIME_ERROR& e) {
+			std::cerr << "ERROR: " << e.what() << std::endl;
+		}
+	Game gm(*window, audio, localization);
+	WebServer server(songs);
 	try {
 		// Load audio samples
 		gm.loading(_("Loading audio samples..."), 0.5);
@@ -161,30 +158,35 @@ void mainLoop(std::string const& songlist) {
 		audio.loadSample("guitar fail4", findFile("sounds/guitar_fail4.ogg"));
 		audio.loadSample("guitar fail5", findFile("sounds/guitar_fail5.ogg"));
 		audio.loadSample("guitar fail6", findFile("sounds/guitar_fail6.ogg"));
+		audio.loadSample("notice.ogg",findFile("notice.ogg"));
 		// Load screens
 		gm.loading(_("Creating screens..."), 0.7);
-		gm.addScreen(new ScreenIntro("Intro", audio));
-		gm.addScreen(new ScreenSongs("Songs", audio, songs, database));
-		gm.addScreen(new ScreenSing("Sing", audio, database, backgrounds));
-		gm.addScreen(new ScreenPractice("Practice", audio));
-		gm.addScreen(new ScreenAudioDevices("AudioDevices", audio));
-		gm.addScreen(new ScreenPaths("Paths", audio));
-		gm.addScreen(new ScreenPlayers("Players", audio, database));
-		gm.addScreen(new ScreenPlaylist("Playlist", audio, songs, backgrounds));
+		gm.addScreen(std::make_unique<ScreenIntro>("Intro", audio));
+		gm.addScreen(std::make_unique<ScreenSongs>("Songs", audio, songs, database));
+		gm.addScreen(std::make_unique<ScreenSing>("Sing", audio, database, backgrounds));
+		gm.addScreen(std::make_unique<ScreenPractice>("Practice", audio));
+		gm.addScreen(std::make_unique<ScreenAudioDevices>("AudioDevices", audio));
+		gm.addScreen(std::make_unique<ScreenPaths>("Paths", audio, songs));
+		gm.addScreen(std::make_unique<ScreenPlayers>("Players", audio, database));
+		gm.addScreen(std::make_unique<ScreenPlaylist>("Playlist", audio, songs, backgrounds));
 		gm.activateScreen("Intro");
-		gm.loading(_("Entering main menu"), 0.8);
+		gm.loading(_("Entering main menu..."), 0.8);
 		gm.updateScreen();  // exit/enter, any exception is fatal error
-		gm.loading(_("Loading complete"), 1.0);
+		gm.loading(_("Loading complete!"), 1.0);
 		// Main loop
-		boost::xtime time = now();
+		auto time = Clock::now();
 		unsigned frames = 0;
 		std::clog << "core/info: Assets loaded, entering main loop." << std::endl;
 		while (!gm.isFinished()) {
 			Profiler prof("mainloop");
 			bool benchmarking = config["graphic/fps"].b();
-			if( g_take_screenshot ) {
+			if (songs.doneLoading == true && songs.displayedAlert == false) {
+				gm.dialog(_("Done Loading!\n Loaded ") + std::to_string(songs.loadedSongs()) + " Songs.");
+				songs.displayedAlert = true;
+			}
+			if (g_take_screenshot) {
 				try {
-					window.screenshot();
+					window->screenshot();
 					gm.flashMessage(_("Screenshot taken!"));
 				} catch (EXCEPTION& e) {
 					std::cerr << "ERROR: " << e.what() << std::endl;
@@ -195,55 +197,59 @@ void mainLoop(std::string const& songlist) {
 			gm.updateScreen();  // exit/enter, any exception is fatal error
 			if (benchmarking) prof("misc");
 			try {
-				window.blank();
+				window->blank();
 				// Draw
-				window.render(boost::bind(&Game::drawScreen, &gm));
+				window->render([&gm]{ gm.drawScreen(); });
 				if (benchmarking) { glFinish(); prof("draw"); }
 				// Display (and wait until next frame)
-				window.swap();
+				window->swap();
 				if (benchmarking) { glFinish(); prof("swap"); }
-				updateSurfaces();
+				updateTextures();
 				gm.prepareScreen();
-				if (benchmarking) { glFinish(); prof("surfaces"); }
+				if (benchmarking) { glFinish(); prof("textures"); }
 				if (benchmarking) {
 					++frames;
-					if (now() - time > 1.0) {
+					if (Clock::now() - time > 1s) {
 						std::ostringstream oss;
 						oss << frames << " FPS";
 						gm.flashMessage(oss.str());
-						time += 1.0;
+						time += 1s;
 						frames = 0;
 					}
 				} else {
-					boost::thread::sleep(time + 0.01); // Max 100 FPS
-					time = now();
+					std::this_thread::sleep_until(time + 10ms); // Max 100 FPS
+					time = Clock::now();
 					frames = 0;
 				}
 				if (benchmarking) prof("fpsctrl");
 				// Process events for the next frame
-				gm.controllers.process(now());
-				checkEvents(gm);
+				auto eventTime = Clock::now();
+				gm.controllers.process(eventTime);
+				checkEvents(gm, eventTime);
 				if (benchmarking) prof("events");
-			} catch (RUNTIME_ERROR& e) {
-				std::cerr << "ERROR: " << e.what() << std::endl;
-				gm.flashMessage(std::string("ERROR: ") + e.what());
+		} catch (RUNTIME_ERROR& e) {
+			std::cerr << "ERROR: " << e.what() << std::endl;
+			gm.flashMessage(std::string("ERROR: ") + e.what());
 			}
 		}
+		writeConfig();
 	} catch (EXCEPTION& e) {
 		std::clog << "core/error: Exiting due to fatal error: " << e.what() << std::endl;
 		gm.fatalError(e.what());  // Notify the user
 		throw;
-	} catch (QuitNow&) {
+		} catch (QuitNow&) {
 		std::cerr << "Terminated." << std::endl;
-	}
+		}
 }
 
 /// Simple test utility to make mapping of joystick buttons/axes easier
 void jstestLoop() {
 	try {
-		Window window(config["graphic/window_width"].i(), config["graphic/window_height"].i(), false);
+		config["graphic/fullscreen"].b() = false;
+		config["graphic/window_width"].i() = 640;
+		config["graphic/window_height"].i() = 360;
+		Window window;
 		// Main loop
-		boost::xtime time = now();
 		int oldjoy = -1, oldaxis = -1, oldvalue = -1;
 		while (true) {
 			SDL_Event e;
@@ -266,8 +272,7 @@ void jstestLoop() {
 				}
 			}
 			window.blank(); window.swap();
-			boost::thread::sleep(time + 0.01); // Max 100 FPS
-			time = now();
+			std::this_thread::sleep_for(10ms); // Max 100 FPS
 		}
 	} catch (EXCEPTION& e) {
 		std::cerr << "ERROR: " << e.what() << std::endl;
@@ -295,7 +300,7 @@ void fatalError(std::string msg, bool hasLog = false, std::string title = "FATAL
 	errMsg << std::endl << "If you think this is a bug in Performous, please report it at "
 	  << std::endl << "  https://github.com/performous/performous/issues";
 	SDL_ShowSimpleMessageBox(SDL_MESSAGEBOX_ERROR, title.c_str(),
-	  errMsg.str().c_str(), NULL);
+	  errMsg.str().c_str(), nullptr);
 	std::cerr << title << ": " << msg << std::endl;
 	if (hasLog) {
 		std::clog << "core/error: " << errMsg.str() << std::endl;
@@ -363,18 +368,13 @@ int main(int argc, char** argv) try {
 			std::clog << "core/notice: Starting audio subsystem for audiohelp (errors printed on console may be ignored)." << std::endl;
 			Audio audio;
 			// Print the devices
-			portaudio::AudioDevices ads;
-			std::cout << ads.dump();
+			std::cout << portaudio::AudioBackends().dump();
 			// Some examples
 			std::cout << "Example --audio parameters" << std::endl;
 			std::cout << "  --audio \"out=2\"         # Pick first working two-channel playback device" << std::endl;
 			std::cout << "  --audio \"dev=1 out=2\"   # Pick device id 1 and assign stereo playback" << std::endl;
 			std::cout << "  --audio 'dev=\"HDA Intel\" mics=blue,red'   # HDA Intel with two mics" << std::endl;
 			std::cout << "  --audio 'dev=pulse out=2 mics=blue'       # PulseAudio with input and output" << std::endl;
-			// Give audio a little time to shutdown but then just quit
-			boost::thread audiokiller(boost::bind(&Audio::close, boost::ref(audio)));
-			if (!audiokiller.timed_join(boost::posix_time::milliseconds(2000)))
-			  std::clog << "core/warning: Closing audio hung for over two seconds." << std::endl;
 			return EXIT_SUCCESS;
 		}
 		// Override XML config for options that were specified from commandline or performous.conf
@@ -404,18 +404,8 @@ int main(int argc, char** argv) try {
 
 void outputOptionalFeatureStatus() {
 	std::clog << "core/notice: " PACKAGE " " VERSION " starting..."
-	  << "\n  Internationalization: " << (Gettext::enabled() ? "Enabled" : "Disabled")
+	  << "\n  Internationalization: " << ("Enabled")
 	  << "\n  MIDI Hardware I/O:    " << (input::Hardware::midiEnabled() ? "Enabled" : "Disabled")
 	  << "\n  Webcam support:       " << (Webcam::enabled() ? "Enabled" : "Disabled")
 	  << std::endl;
 }
-
-#if defined(_WIN32)
-// Force high-performance graphics on dual-GPU systems
-extern "C" {
-	// http://developer.download.nvidia.com/devzone/devcenter/gamegraphics/files/OptimusRenderingPolicies.pdf
-	__declspec(dllexport) DWORD NvOptimusEnablement = 0x00000001;
-	// https://community.amd.com/thread/169965
-	__declspec(dllexport) int AmdPowerXpressRequestHighPerformance = 1;
-}
-#endif
