@@ -1,12 +1,12 @@
 #include "screen_audiodevices.hh"
 
+#include "audio.hh"
 #include "configuration.hh"
 #include "controllers.hh"
+#include "platform.hh"
 #include "theme.hh"
-#include "audio.hh"
 #include "i18n.hh"
-#include <boost/thread.hpp>
-#include <boost/bind.hpp>
+#include "game.hh"
 
 namespace {
 	static const int unassigned_id = -1;  // mic.dev value for unassigned
@@ -20,15 +20,24 @@ namespace {
 	}
 }
 
+int getBackend() {
+	static std::string selectedBackend = Audio::backendConfig().getValue();
+	return PaHostApiNameToHostApiTypeId(selectedBackend);
+}
+
+
 ScreenAudioDevices::ScreenAudioDevices(std::string const& name, Audio& audio): Screen(name), m_audio(audio) {
-	m_selector.reset(new Surface(findFile("device_selector.svg")));
-	m_mic_icon.reset(new Surface(findFile("sing_pbox.svg")));
-	m_pdev_icon.reset(new Surface(findFile("icon_pdev.svg")));
+	m_selector = std::make_unique<Texture>(findFile("device_selector.svg"));
+	m_mic_icon = std::make_unique<Texture>(findFile("sing_pbox.svg"));
+	m_pdev_icon = std::make_unique<Texture>(findFile("icon_pdev.svg"));
 }
 
 void ScreenAudioDevices::enter() {
-	m_theme.reset(new ThemeAudioDevices());
-	portaudio::AudioDevices ads;
+	int bend = getBackend();
+	std::clog << "audio-devices/debug: Entering audio Devices... backend has been detected as: " << bend << std::endl;
+	m_theme = std::make_unique<ThemeAudioDevices>();
+	PaHostApiTypeId backend = PaHostApiTypeId(bend);
+	portaudio::AudioDevices ads(backend);
 	m_devs = ads.devices;
 	// FIXME: Something more elegant, like a warning box
 	if (m_devs.empty()) throw std::runtime_error("No audio devices found!");
@@ -44,9 +53,12 @@ void ScreenAudioDevices::enter() {
 			if (!countRow("green", *it, countmap["green"])) { ok = false; break; }
 			if (!countRow("yellow", *it, countmap["yellow"])) { ok = false; break; }
 			if(!countRow("fuchsia", *it, countmap["fuchsia"])) { ok = false; break; }
-			if(!countRow("lightgreen", *it, countmap["lightgreen"])) { ok = false; break; }
+			if(!countRow("orange", *it, countmap["orange"])) { ok = false; break; }
 			if(!countRow("purple", *it, countmap["purple"])) { ok = false; break; }
 			if(!countRow("aqua", *it, countmap["aqua"])) { ok = false; break; }
+			if(!countRow("white", *it, countmap["white"])) { ok = false; break; }
+			if(!countRow("gray", *it, countmap["gray"])) { ok = false; break; }
+			if(!countRow("black", *it, countmap["black"])) { ok = false; break; }
 			if (!countRow("out=", *it, countmap["out="])) { ok = false; break; }
 		}
 		if (!ok)
@@ -68,14 +80,14 @@ void ScreenAudioDevices::manageEvent(input::NavEvent const& event) {
 	input::NavButton nav = event.button;
 	auto& chpos = m_channels[m_selected_column].pos;
 	const unsigned posN = m_devs.size() + 1;
-	if (nav == input::NAV_CANCEL) gm->activateScreen("Intro");
-	else if (nav == input::NAV_PAUSE) m_audio.togglePause();
+	if (nav == input::NavButton::CANCEL) gm->activateScreen("Intro");
+	else if (nav == input::NavButton::PAUSE) m_audio.togglePause();
 	else if (m_devs.empty()) return; // The rest work if there are any devices
-	else if (nav == input::NAV_START) { if (save()) gm->activateScreen("Intro"); }
-	else if (nav == input::NAV_LEFT && m_selected_column > 0) --m_selected_column;
-	else if (nav == input::NAV_RIGHT && m_selected_column < m_channels.size()-1) ++m_selected_column;
-	else if (nav == input::NAV_UP) chpos = (chpos + posN) % posN - 1;
-	else if (nav == input::NAV_DOWN) chpos = (chpos + posN + 2) % posN - 1;
+	else if (nav == input::NavButton::START) { if (save()) gm->activateScreen("Intro"); }
+	else if (nav == input::NavButton::LEFT && m_selected_column > 0) --m_selected_column;
+	else if (nav == input::NavButton::RIGHT && m_selected_column < m_channels.size()-1) ++m_selected_column;
+	else if (nav == input::NavButton::UP) chpos = (chpos + posN) % posN - 1;
+	else if (nav == input::NavButton::DOWN) chpos = (chpos + posN + 2) % posN - 1;
 }
 
 void ScreenAudioDevices::manageEvent(SDL_Event event) {
@@ -84,7 +96,7 @@ void ScreenAudioDevices::manageEvent(SDL_Event event) {
 		uint16_t modifier = event.key.keysym.mod;
 		if (m_devs.empty()) return; // The rest work if there are any config options
 		// Reset to defaults
-		else if (key == SDL_SCANCODE_R && modifier & KMOD_CTRL) {
+		else if (key == SDL_SCANCODE_R && modifier & Platform::shortcutModifier()) {
 			config["audio/devices"].reset(modifier & KMOD_ALT);
 			save(true); // Save to disk, reload audio & reload UI to keep stuff consistent
 		}
@@ -98,23 +110,27 @@ void ScreenAudioDevices::draw() {
 	const float xstep = (xoff - 0.5 + xoff) / m_channels.size();
 	const float ystep = yoff*2 / m_devs.size();
 	// Device text & bg
-	m_theme->device_bg.dimensions.stretch(std::abs(xoff*2), m_mic_icon->dimensions.h()*0.9).middle();
+	m_theme->device_bg.dimensions.stretch(std::abs(xoff*2.15), m_mic_icon->dimensions.h()*0.9).middle();
 	m_selector->dimensions.stretch(m_mic_icon->dimensions.w() * 1.75, m_mic_icon->dimensions.h() * 1.75);
-	for (size_t i = 0; i <= m_devs.size(); ++i) {
+	for (size_t i = 0; i < m_devs.size(); ++i) {
 		const float y = -yoff + i*ystep;
 		float alpha = 1.0f;
-		// "Grey out" devices that doesn't fit the selection
-		if (m_channels[m_selected_column].name == "OUT" && !m_devs[i].out) alpha = 0.5f;
-		else if (m_channels[m_selected_column].name != "OUT" && !m_devs[i].in) alpha = 0.5f;
+
+		const bool isDevice = (i < m_devs.size());
+		if (isDevice) {
+			// "Grey out" devices that doesn't fit the selection
+			if (m_channels[m_selected_column].name == "OUT" && !m_devs[i].out) alpha = 0.5f;
+			else if (m_channels[m_selected_column].name != "OUT" && !m_devs[i].in) alpha = 0.5f;
+		}
 		m_theme->device_bg.dimensions.center(y);
 		m_theme->device_bg.draw();
 		ColorTrans c(Color::alpha(alpha));
 		m_theme->device.dimensions.middle(-xstep*0.5).center(y);
-		m_theme->device.draw(i < m_devs.size() ? m_devs[i].desc() : _("- Unassigned -"));
+		m_theme->device.draw(isDevice ? m_devs[i].desc() : _("- Unassigned -"));
 	}
 	// Icons
 	for (size_t i = 0; i < m_channels.size(); ++i) {
-		Surface& srf = (i < m_channels.size()-1) ? *m_mic_icon : *m_pdev_icon;
+		Texture& srf = (i < m_channels.size()-1) ? *m_mic_icon : *m_pdev_icon;
 		{
 			ColorTrans c(MicrophoneColor::get(m_channels[i].name));
 			int pos = m_channels[i].pos;
@@ -140,14 +156,14 @@ void ScreenAudioDevices::draw() {
 }
 
 void ScreenAudioDevices::load() {
-	std::string names[] = { "blue", "red", "green", "yellow", "fuchsia", "lightgreen", "purple", "aqua", "OUT" }; //there were 4 colors here
+	std::string names[] = { "blue", "red", "green", "yellow", "fuchsia", "orange", "purple", "aqua", "white", "gray", "black", "OUT" }; //there were 4 colors here
 	m_channels.assign(std::begin(names), std::end(names));
 	// Get the currently assigned devices for each channel (FIXME: this is a really stupid algorithm)
 	for (auto const& d: m_audio.devices()) {
 		for (auto& c: m_channels) {
 			if (!d.isChannel(c.name)) continue;
 			for (size_t i = 0; i < m_devs.size(); ++i) {
-				if (unsigned(m_devs[i].idx) == d.dev) c.pos = i;
+				if (unsigned(m_devs[i].index) == d.dev) c.pos = i;
 			}
 		}
 	}
@@ -175,11 +191,7 @@ bool ScreenAudioDevices::save(bool skip_ui_config) {
 		}
 		config["audio/devices"].sl() = devconf;
 	}
-	writeConfig(); // Save the new config
-	// Give audio a little time to shutdown but then just quit
-	boost::thread audiokiller(boost::bind(&Audio::close, boost::ref(m_audio)));
-	if (!audiokiller.timed_join(boost::posix_time::milliseconds(2500)))
-		Game::getSingletonPtr()->fatalError("Audio hung for some reason.\nPlease restart Performous.");
+	writeConfig(false); // Save the new config
 	m_audio.restart(); // Reload audio to take the new settings into use
 	m_audio.playMusic(findFile("menu.ogg"), true); // Start music again
 	// Check that all went well
