@@ -1,4 +1,4 @@
-#include "screen_playlist.hh"
+﻿#include "screen_playlist.hh"
 #include "menu.hh"
 #include "screen_sing.hh"
 #include "playlist.hh"
@@ -8,14 +8,9 @@
 
 #include <iostream>
 #include <sstream>
-#include <boost/format.hpp>
-
-namespace {
-	static const double NEXT_TIMEOUT = 15.0; // go to next song in 15 seconds
-}
 
 ScreenPlaylist::ScreenPlaylist(std::string const& name,Audio& audio, Songs& songs, Backgrounds& bgs):
-	Screen(name), m_audio(audio), m_songs(songs), m_backgrounds(bgs), m_covers(20), keyPressed()
+	Screen(name), m_audio(audio), m_songs(songs), m_backgrounds(bgs), keyPressed()
 {}
 
 void ScreenPlaylist::enter() {
@@ -24,26 +19,23 @@ void ScreenPlaylist::enter() {
 	gm->loading(_("Initializing webcam..."), 0.1);
 	if (config["graphic/webcam"].b() && Webcam::enabled()) {
 		try {
-			m_cam.reset(new Webcam(config["graphic/webcamid"].i()));
+			m_cam = std::make_unique<Webcam>(config["graphic/webcamid"].i());
 		} catch (std::exception& e) { std::cout << e.what() << std::endl; };
 	}
 	m_audio.togglePause();
 	if (gm->getCurrentPlayList().isEmpty()) {
-		if(config["game/autoplay"].b()) {
-			m_songs.setFilter("");
-			Screen* s = gm->getScreen("Sing");
-			ScreenSing* ss = dynamic_cast<ScreenSing*> (s);
-			assert(ss);
-			int randomsong = std::rand() % m_songs.size();
-			ss->setSong(m_songs[randomsong]);
-			gm->activateScreen("Sing");
-		} else {
+		if(!config["game/autoplay"].b()) {
 			gm->activateScreen("Songs");
 		}
 	}
 	keyPressed = false;
-	m_nextTimer.setValue(NEXT_TIMEOUT);
+	auto timervalue = config["game/playlist_screen_timeout"].i();
+	if(timervalue < 0.0) {
+		timervalue = 0.0;
+	}
+	m_nextTimer.setValue(timervalue);
 	overlay_menu.close();
+	gm->loading(_("Loading song timestamps..."), 0.2);
 	createSongListMenu();
 	songlist_menu.open();
 	reloadGL();
@@ -54,12 +46,12 @@ void ScreenPlaylist::prepare() {
 }
 
 void ScreenPlaylist::reloadGL() {
-	theme.reset(new ThemePlaylistScreen());
-	m_menuTheme.reset(new ThemeInstrumentMenu());
-	m_singCover.reset(new Surface(findFile("no_cover.svg")));
-	m_instrumentCover.reset(new Surface(findFile("instrument_cover.svg")));
-	m_bandCover.reset(new Surface(findFile("band_cover.svg")));
-	m_danceCover.reset(new Surface(findFile("dance_cover.svg")));
+	theme = std::make_unique<ThemePlaylistScreen>();
+	m_menuTheme = std::make_unique<ThemeInstrumentMenu>();
+	m_singCover = std::make_unique<Texture>(findFile("no_cover.svg"));
+	m_instrumentCover = std::make_unique<Texture>(findFile("instrument_cover.svg"));
+	m_bandCover = std::make_unique<Texture>(findFile("band_cover.svg"));
+	m_danceCover = std::make_unique<Texture>(findFile("dance_cover.svg"));
 }
 
 void ScreenPlaylist::exit() {
@@ -80,7 +72,7 @@ void ScreenPlaylist::manageEvent(input::NavEvent const& event) {
 	if (keyPressed == false)
 		keyPressed = true;
 
-	if (nav == input::NAV_CANCEL) {
+	if (nav == input::NavButton::CANCEL) {
 		if(overlay_menu.isOpen()) {
 			overlay_menu.close();
 		} else {
@@ -88,17 +80,17 @@ void ScreenPlaylist::manageEvent(input::NavEvent const& event) {
 			overlay_menu.open();
 		}
 	} else {
-		if (nav == input::NAV_PAUSE) {
+		if (nav == input::NavButton::PAUSE) {
 			m_audio.togglePause();
-		} else if (nav == input::NAV_START) {
+		} else if (nav == input::NavButton::START) {
 			menu.action();
-		} else if (event.menu == input::NAVMENU_A_PREV) {
+		} else if (event.menu == input::NavMenu::A_PREV) {
 			menu.move(-1);
-		} else if (event.menu == input::NAVMENU_A_NEXT) {
+		} else if (event.menu == input::NavMenu::A_NEXT) {
 			menu.move(1);
-		} else if (event.menu == input::NAVMENU_B_PREV) {
+		} else if (event.menu == input::NavMenu::B_PREV) {
 			menu.move(-1);
-		} else if (event.menu == input::NAVMENU_B_NEXT) {
+		} else if (event.menu == input::NavMenu::B_NEXT) {
 			menu.move(1);
 		}
 	}
@@ -110,13 +102,19 @@ void ScreenPlaylist::manageEvent(SDL_Event) {
 
 void ScreenPlaylist::draw() {
 	Game* gm = Game::getSingletonPtr();
-	if (!m_background || m_background->empty()) m_background.reset(new Surface(m_backgrounds.getRandom()));
+	if (!m_background || m_background->empty()) m_background = std::make_unique<Texture>(m_backgrounds.getRandom());
 	m_background->draw();
 	if (m_nextTimer.get() == 0.0 && keyPressed == false) {
 		Screen* s = gm->getScreen("Sing");
 		ScreenSing* ss = dynamic_cast<ScreenSing*> (s);
 		assert(ss);
-		ss->setSong(gm->getCurrentPlayList().getNext());
+		if(gm->getCurrentPlayList().isEmpty()) {
+			m_songs.setFilter("");
+			auto randomsong = std::rand() % m_songs.size();
+			ss->setSong(m_songs[randomsong]);
+		} else {
+			ss->setSong(gm->getCurrentPlayList().getNext());
+		}
 		gm->activateScreen("Sing");
 	}
 	if (m_cam && config["graphic/webcam"].b()) m_cam->render();
@@ -125,27 +123,44 @@ void ScreenPlaylist::draw() {
 	if (overlay_menu.isOpen()) {
 		drawMenu();
 	}
+	if(needsUpdate) {
+		std::lock_guard<std::mutex> l(m_mutex);
+		createSongListMenu();
+		needsUpdate = false;
+	}
 	auto const& playlist = gm->getCurrentPlayList().getList();
 	for (unsigned i = playlist.size() - 1; i < playlist.size(); --i) {
 		if(i < 9) { //only draw the first 9 covers
-			Surface& s = getCover(*playlist[i]);
-			double pos =  i / std::max<double>(9, 9);
+			Texture& s = getCover(*playlist[i]);
+			float pos =  i / std::max<float>(9, 9);
 			using namespace glmath;
 			Transform trans(
-			  translate(vec3(-0.4 + 0.9 * pos, 0.045)) //vec3(horizontal-offset-from-center, vertical offset from screen_bottom)
-			  * rotate(-0.0, vec3(0.0, 1.0, 0.0))
+			  translate(vec3(-0.4f + 0.9f * pos, 0.045f, 0.0f)) //vec3(horizontal-offset-from-center, vertical offset from screen_bottom)
+			  * rotate(-0.0f, vec3(0.0f, 1.0f, 0.0f))
 			);
 			s.dimensions.middle().screenBottom(-0.06).fitInside(0.08, 0.08);
 			s.draw();
 		}
 	}
 }
-Surface& ScreenPlaylist::getCover(Song const& song) {
-	Surface* cover = nullptr;
+
+Texture* ScreenPlaylist::loadTextureFromMap(fs::path path) {
+	if(m_covers.find(path) == m_covers.end()) {
+		std::pair<fs::path, std::unique_ptr<Texture>> kv = std::make_pair(path, std::make_unique<Texture>(path));
+		m_covers.insert(std::move(kv));
+	}
+	try {
+		return m_covers.at(path).get();
+	} catch (std::exception const&) {}
+	return nullptr;
+}
+
+Texture& ScreenPlaylist::getCover(Song const& song) {
+	Texture* cover = nullptr;
 	// Fetch cover image from cache or try loading it
-	if (!song.cover.empty()) try { cover = &m_covers[song.cover]; } catch (std::exception const&) {}
+	if (!song.cover.empty()) cover = loadTextureFromMap(song.cover);
 	// Fallback to background image as cover if needed
-	if (!cover && !song.background.empty()) try { cover = &m_covers[song.background]; } catch (std::exception const&) {}
+	if (!cover && !song.background.empty()) cover = loadTextureFromMap(song.background);
 	// Use empty cover
 	if (!cover) {
 		if(song.hasDance()) {
@@ -168,7 +183,13 @@ void ScreenPlaylist::createEscMenu() {
 		Screen* s = gm->getScreen("Sing");
 		ScreenSing* ss = dynamic_cast<ScreenSing*> (s);
 		assert(ss);
-		ss->setSong(gm->getCurrentPlayList().getNext());
+		if(!gm->getCurrentPlayList().isEmpty()) {
+			ss->setSong(gm->getCurrentPlayList().getNext());
+		} else {
+			m_songs.setFilter("");
+			auto randomsong = std::rand() % m_songs.size();
+			ss->setSong(m_songs[randomsong]);
+		}
 		gm->activateScreen("Sing");
 	}));
 	overlay_menu.add(MenuOption(_("Add songs"), _("Open the song browser to add more songs")).screen("Songs"));
@@ -255,7 +276,7 @@ void ScreenPlaylist::draw_menu_options() {
 			}
 			wcounter = std::max(wcounter, theme->option_selected.w() + 2 * sel_margin); // Calculate the widest entry
 			// If this is a config item, show the value below
-			if (opt.type == MenuOption::CHANGE_VALUE) {
+			if (opt.type == MenuOption::Type::CHANGE_VALUE) {
 				++ii; // Use a slot for the value
 				theme->option_selected.dimensions.left(x + sel_margin).center(-0.1 + (selanim+1)*0.08);
 				theme->option_selected.draw("<  " + opt.value->getValue() + "  >");
@@ -275,8 +296,10 @@ void ScreenPlaylist::draw_menu_options() {
 }
 
 SvgTxtTheme& ScreenPlaylist::getTextObject(std::string const& txt) {
-	if (theme->options.contains(txt)) return theme->options[txt];
-	return *theme->options.insert(txt, new SvgTxtTheme(findFile("mainmenu_option.svg"), config["graphic/text_lod"].f()))->second;
+	if (theme->options.find(txt) != theme->options.end()) return (*theme->options.at(txt).get());
+	std::pair<std::string, std::unique_ptr<SvgTxtTheme>> kv = std::make_pair(txt, std::make_unique<SvgTxtTheme>(findFile("mainmenu_option.svg"), config["graphic/text_lod"].f()));
+	theme->options.insert(std::move(kv));
+	return (*theme->options.at(txt).get());
 }
 
 void ScreenPlaylist::createSongListMenu() {
@@ -285,18 +308,38 @@ void ScreenPlaylist::createSongListMenu() {
 	int count = 1;
 	songlist_menu.clear();
 	SongList& currentList = gm->getCurrentPlayList().getList();
+	int totaldurationSeconds = 0;
 	for (auto const& song: currentList) {
-		oss_playlist << "#" << count << " : " << song->artist << " - " << song->title;
+		//timestamp handles
+		int hours = 0;
+		int minutes = 0;
+		int seconds = totaldurationSeconds;
+		while(seconds >= 60) {
+			minutes++;
+			seconds -= 60;
+		}
+		while(minutes >= 60) {
+			hours ++;
+			minutes -=60;
+		}
+		oss_playlist << "#" << count << " : " << song->artist << " - " << song->title << "  +";
+		if(hours > 0) {
+			oss_playlist << std::setw(2) << std::setfill('0') << hours << ":";
+		}
+			oss_playlist << std::setw(2) << std::setfill('0') << minutes << ":" << std::setw(2) << std::setfill('0') << seconds;
 		std::string songinfo = oss_playlist.str();
 		if (songinfo.length() > 20) {
-			songinfo = songinfo + "                          >"; //FIXME: ugly hack to make the text scale so it fits on screen!
+			songinfo = songinfo + "                           >"; //FIXME: ugly hack to make the text scale so it fits on screen!
 		}
+		//then add it to the menu:
 		songlist_menu.add(MenuOption(_(songinfo.c_str()),_("Press enter to view song options")).call([this, count]() {
 			createSongMenu(count);
 			overlay_menu.open();
 		}));
 		oss_playlist.str("");
 		count++;
+		totaldurationSeconds += song->getDurationSeconds();
+		totaldurationSeconds += config["game/playlist_screen_timeout"].i();
 	}
 	songlist_menu.add(MenuOption(_("View more options"),_("View general playlist settings")).call([this]() {
 		createEscMenu();
@@ -310,7 +353,7 @@ void ScreenPlaylist::createSongMenu(int songNumber) {
 	std::string firstDesc = songNumber >= 2 ?
 		_("Ignore the playlist's order and play this song first") :
 		_("Start the song already!");
-	overlay_menu.add(MenuOption(firstOption, firstDesc).call([this, songNumber]() {
+	overlay_menu.add(MenuOption(firstOption, firstDesc).call([songNumber]() {
 		Game* gm = Game::getSingletonPtr();
 		Screen* s = gm->getScreen("Sing");
 		ScreenSing* ss = dynamic_cast<ScreenSing*>(s);
@@ -340,7 +383,7 @@ void ScreenPlaylist::createSongMenu(int songNumber) {
 	Game* gm = Game::getSingletonPtr();
 	int size = gm->getCurrentPlayList().getList().size();
 	if (songNumber < size) { //can't move down the last song
-		overlay_menu.add(MenuOption(_("Move Down"), _("Move this song down the list")).call([this, songNumber]() {
+		overlay_menu.add(MenuOption(_("Move down"), _("Move this song down the list")).call([this, songNumber]() {
 			Game* gm = Game::getSingletonPtr();
 			gm->getCurrentPlayList().swap(songNumber -1, songNumber);
 			createSongListMenu();
@@ -351,3 +394,8 @@ void ScreenPlaylist::createSongMenu(int songNumber) {
 		overlay_menu.close();
 	}));
 }
+
+void ScreenPlaylist::triggerSongListUpdate() {
+std::lock_guard<std::mutex> l (m_mutex);
+needsUpdate = true;
+}
