@@ -10,23 +10,22 @@
 #include "i18n.hh"
 #include "layout_singer.hh"
 #include "menu.hh"
+#include "platform.hh"
 #include "screen_players.hh"
 #include "songparser.hh"
 #include "util.hh"
 #include "video.hh"
 #include "webcam.hh"
 #include "screen_songs.hh"
+#include "notegraphscalerfactory.hh"
 
 #include <boost/format.hpp>
-#include <boost/lexical_cast.hpp>
 #include <iostream>
 #include <stdexcept>
 #include <cmath>
 #include <utility>
 
 namespace {
-	static const double QUIT_TIMEOUT = 20.0; // Return to songs screen after 20 seconds in score screen
-
 	/// Add a flash message about the state of a config item
 	void dispInFlash(ConfigItem& ci) {
 		Game* gm = Game::getSingletonPtr();
@@ -47,15 +46,14 @@ void ScreenSing::enter() {
 	gm->loading(_("Initializing webcam..."), 0.1);
 	if (config["graphic/webcam"].b() && Webcam::enabled()) {
 		try {
-			m_cam.reset(new Webcam(config["graphic/webcamid"].i()));
+			m_cam = std::make_unique<Webcam>(config["graphic/webcamid"].i());
 		} catch (std::exception& e) { std::cout << e.what() << std::endl; };
 	}
 	// Load video
 	gm->loading(_("Loading video..."), 0.2);
 	if (!m_song->video.empty() && config["graphic/video"].b()) {
-		m_video.reset(new Video(m_song->video, m_song->videoGap));
+		m_video = std::make_unique<Video>(m_song->video, m_song->videoGap);
 	}
-	boost::ptr_vector<Analyzer>& analyzers = m_audio.analyzers();
 	reloadGL();
 	// Load song notes
 	gm->loading(_("Loading song..."), 0.4);
@@ -72,51 +70,77 @@ void ScreenSing::enter() {
 	m_audio.playMusic(m_song->music, false, 0.0, setup_delay);
 	gm->loading(_("Loading menu..."), 0.7);
 	{
-		VocalTracks const& tracks = m_song->vocalTracks;
-		unsigned players = analyzers.size();
-		m_menu.clear();
-		m_menu.add(MenuOption(_("Start"), _("Start performing")).call(std::bind(&ScreenSing::setupVocals, this)));
 		m_duet = ConfigItem(0);
-		if (players == 0) players = 1;  // No mic? We display lyrics anyway
-		if (players > 1) { // Duet toggle
-			m_duet.addEnum(_("Duet mode"));
-			m_duet.addEnum(_("Normal mode"));
-			m_menu.add(MenuOption("", _("Switch between duet and regular singing mode")).changer(m_duet));
-		}
-		// Add vocal track selector for each player
-		for (unsigned player = 0; player < players; ++player) {
+		for (size_t player = 0; player < players(); ++player) {
 			ConfigItem& vocalTrack = m_vocalTracks[player];
 			vocalTrack = ConfigItem(0);
-			for (auto const& track: tracks) vocalTrack.addEnum(track.second.name);
-			if (tracks.size() > 1 && player % 2) ++vocalTrack;  // Every other player gets the second track
-			m_menu.add(MenuOption("", _("Change vocal track")).changer(vocalTrack));
 		}
-		m_menu.add(MenuOption(_("Quit"), _("Exit to song browser")).screen("Songs"));
-		m_menu.open();
-		if (tracks.size() <= 1) setupVocals();  // No duet menu
+		prepareVoicesMenu();
 	}
 	gm->showLogo(false);
 	gm->loading(_("Loading complete"), 1.0);
 }
 
+void ScreenSing::prepareVoicesMenu(size_t moveSelectionTo) {
+		VocalTracks const& tracks = m_song->vocalTracks;
+		m_menu.clear();
+		m_menu.add(MenuOption(_("Start"), _("Start performing")).call([this]{ setupVocals(); }));
+
+		if (players() > 1) { // Duet toggle
+			m_duet.addEnum(_("Duet mode"));
+			m_duet.addEnum(_("Normal mode"));
+			m_menu.add(MenuOption("", _("Switch between duet and regular singing mode")).changer(m_duet,"song/duet"));
+		}
+		// Add vocal track selector for each player
+		for (size_t player = 0; player < players(); ++player) {
+			ConfigItem& vocalTrack = m_vocalTracks[player];
+			for (auto const& track: tracks) vocalTrack.addEnum(track.second.name);
+			if (tracks.size() > 1) {
+				if (player % 2) vocalTrack.selectEnum(m_song->getVocalTrack(SongParserUtil::DUET_P2).name);  // Every other player gets the second track
+				else vocalTrack.selectEnum(m_song->getVocalTrack(TrackName::LEAD_VOCAL).name);
+			}
+			m_menu.add(MenuOption("", _("Change vocal track")).changer(vocalTrack));
+			if (m_duet.i() == 1) {
+				vocalTrack.selectEnum(m_song->getVocalTrack(SongParserUtil::DUET_BOTH).name);
+				break; // If duet mode is disabled, the vocal track selection for players beyond the first is ignored anyway.
+			}
+		}
+		m_menu.add(MenuOption(_("Quit"), _("Exit to song browser")).screen("Songs"));
+		m_menu.select(moveSelectionTo);
+		m_menu.open();
+		if (tracks.size() <= 1) setupVocals();  // No duet menu
+}
+
 void ScreenSing::setupVocals() {
 	if (!m_song->vocalTracks.empty()) {
 		m_layout_singer.clear();
 		Engine::VocalTrackPtrs selectedTracks;
-		boost::ptr_vector<Analyzer>& analyzers = m_audio.analyzers();
-		unsigned players = (analyzers.empty() ? 1 : analyzers.size());  // Always at least 1; should be number of mics
+		auto& analyzers = m_audio.analyzers();
+		//size_t players = (analyzers.empty() ? 1 : analyzers.size());  // Always at least 1; should be number of mics
 		std::set<VocalTrack*> shownTracks;  // Tracks to be included in layout_singer (stored by name for proper sorting and merging duplicates)
-		for (unsigned player = 0; player < players; ++player) {
+		for (size_t player = 0; player < players(); ++player) {
 			VocalTrack* vocal = &m_song->getVocalTrack(m_vocalTracks[(m_duet.i() == 0 ? player : 0)].i());
 			selectedTracks.push_back(vocal);
 			shownTracks.insert(vocal);
 		}
-		//if (shownTracks.size() > 2) throw std::runtime_error("Too many tracks chosen. Only two vocal tracks can be used simultaneously.");
-		for (auto const& trk: shownTracks) m_layout_singer.push_back(new LayoutSinger(*trk, m_database, theme));
+
+		//if (shownTracks.size() > 2) throw std::runtime_error("Too many tracks chosen. Only two vocal tracks can be used simultaneously.")
+		for (auto const& trk: shownTracks) {
+			const auto scaler = NoteGraphScalerFactory().create(*trk);
+			auto layoutSingerPtr = std::unique_ptr<LayoutSinger>(std::make_unique<LayoutSinger>(*trk, m_database, scaler, theme));
+			m_layout_singer.push_back(std::move(layoutSingerPtr));
+		}
 		// Note: Engine maps tracks with analyzers 1:1. If user doesn't have mics, we still want to have singer layout enabled but without engine...
-		if (!analyzers.empty()) m_engine.reset(new Engine(m_audio, selectedTracks, m_database));
+		if (!analyzers.empty()) m_engine = std::make_unique<Engine>(m_audio, selectedTracks, m_database);
 	}
 	createPauseMenu();
+	bool sameVoice = true;
+	for (size_t player = 0; player < players(); ++player) {
+		ConfigItem& vocalTrack = m_vocalTracks[player];
+		if (player == 0) { m_selectedVocal = vocalTrack.i(); }
+		if (vocalTrack.i() != static_cast<int>(m_selectedVocal)) { sameVoice = false; break; }
+	}
+	m_singingDuet = (m_song->hasDuet() && m_duet.i() == 0 && players() > 1 && sameVoice != true);
 	m_audio.pause(false);
 }
 
@@ -124,7 +148,10 @@ void ScreenSing::createPauseMenu() {
 	m_menu.clear();
 	m_menu.add(MenuOption(_("Resume"), _("Back to performing!")));
 	m_menu.add(MenuOption(_("Restart"), _("Start the song\nfrom the beginning")).screen("Sing"));
-	m_menu.add(MenuOption(_("Skip"), _("Skip current song")).screen("Playlist"));
+	Game* gm = Game::getSingletonPtr();
+	if(!gm->getCurrentPlayList().isEmpty() || config["game/autoplay"].b()){
+		m_menu.add(MenuOption(_("Skip"), _("Skip current song")).screen("Playlist"));
+	}
 	m_menu.add(MenuOption(_("Quit"), _("Exit to song browser")).call([]() {
 		Game* gm = Game::getSingletonPtr();
 		gm->activateScreen("Songs");
@@ -134,18 +161,18 @@ void ScreenSing::createPauseMenu() {
 
 void ScreenSing::reloadGL() {
 	// Load UI graphics
-	theme.reset(new ThemeSing());
-	m_menuTheme.reset(new ThemeInstrumentMenu());
-	m_pause_icon.reset(new Surface(findFile("sing_pause.svg")));
-	m_player_icon.reset(new Surface(findFile("sing_pbox.svg"))); // For duet menu
-	m_help.reset(new Surface(findFile("instrumenthelp.svg")));
-	m_progress.reset(new ProgressBar(findFile("sing_progressbg.svg"), findFile("sing_progressfg.svg"), ProgressBar::HORIZONTAL, 0.01f, 0.01f, true));
+	theme = std::make_shared<ThemeSing>();
+	m_menuTheme = std::make_unique<ThemeInstrumentMenu>();
+	m_pause_icon = std::make_unique<Texture>(findFile("sing_pause.svg"));
+	m_player_icon = std::make_unique<Texture>(findFile("sing_pbox.svg")); // For duet menu
+	m_help = std::make_unique<Texture>(findFile("instrumenthelp.svg"));
+	m_progress = std::make_unique<ProgressBar>(findFile("sing_progressbg.svg"), findFile("sing_progressfg.svg"), ProgressBar::Mode::HORIZONTAL, 0.01f, 0.01f, true);
 	// Load background
-	if (!m_song->background.empty()) m_background.reset(new Surface(m_song->background));
+	if (!m_song->background.empty()) m_background = std::make_unique<Texture>(m_song->background);
 }
 
 void ScreenSing::exit() {
-    Game::getSingletonPtr()->controllers.enableEvents(false);
+	Game::getSingletonPtr()->controllers.enableEvents(false);
 	m_engine.reset();
 	m_score_window.reset();
 	m_menu.clear();
@@ -160,6 +187,7 @@ void ScreenSing::exit() {
 	m_song->dropNotes();
 	m_menuTheme.reset();
 	theme.reset();
+	m_audio.fadeout(0);
 	if (m_audio.isPaused()) m_audio.togglePause();
 	Game::getSingletonPtr()->showLogo();
 }
@@ -172,12 +200,12 @@ void ScreenSing::instrumentLayout(double time) {
 	int count_alive = 0, count_menu = 0, i = 0;
 	// Remove dead instruments and do the counting
 	for (Instruments::iterator it = m_instruments.begin(); it != m_instruments.end(); ) {
-		if (it->dead()) {
+		if ((*it)->dead()) {
 			it = m_instruments.erase(it);
 			continue;
 		}
 		++count_alive;
-		if (it->menuOpen()) ++count_menu;
+		if ((*it)->menuOpen()) ++count_menu;
 		++it;
 	}
 	if (count_alive > 0) {
@@ -194,17 +222,17 @@ void ScreenSing::instrumentLayout(double time) {
 	std::map<std::string, CountSum> volume; // Stream id to (count, sum)
 	std::map<std::string, CountSum> pitchbend; // Stream id to (count, sum)
 	for (Instruments::iterator it = m_instruments.begin(); it != m_instruments.end(); ++it, ++i) {
-		it->engine();
-		it->position((0.5 + i - 0.5 * count_alive) * iw, iw); // Do layout stuff
-		it->draw(time);
+		(*it)->engine();
+		(*it)->position((0.5 + i - 0.5 * count_alive) * iw, iw); // Do layout stuff
+		(*it)->draw(time);
 		{
-			CountSum& cs = volume[it->getTrack()];
+			CountSum& cs = volume[(*it)->getTrack()];
 			cs.first++;
-			cs.second += it->correctness();
+			cs.second += (*it)->correctness();
 		}{
-			CountSum& cs = pitchbend[it->getTrack()];
+			CountSum& cs = pitchbend[(*it)->getTrack()];
 			cs.first++;
-			cs.second += it->getWhammy();
+			cs.second += (*it)->getWhammy();
 		}
 	}
 	// Set volume levels (averages of all instruments playing that track)
@@ -233,7 +261,7 @@ void ScreenSing::activateNextScreen()
 	m_database.addSong(m_song);
 	if (m_database.scores.empty() || !m_database.reachedHiscore(m_song)) {
 		// if no highscore reached..
-	    gm->activateScreen("Playlist");
+		gm->activateScreen("Playlist");
 	}
 
 	// Score window visible -> Enter quits to Players Screen
@@ -251,17 +279,18 @@ void ScreenSing::activateNextScreen()
 void ScreenSing::manageEvent(input::NavEvent const& event) {
 	keyPressed = true;
 	input::NavButton nav = event.button;
-	m_quitTimer.setValue(QUIT_TIMEOUT);
+	m_quitTimer.setValue(config["game/results_timeout"].i());
 	double time = m_audio.getPosition();
-	Song::Status status = m_song->status(time);
+	Song::Status status = m_song->status(time, this);
 	// When score window is displayed
 	if (m_score_window.get()) {
-		if (nav == input::NAV_START || nav == input::NAV_CANCEL) activateNextScreen();
+		if (nav == input::NavButton::START || nav == input::NavButton::CANCEL) activateNextScreen();
 		return;  // The rest are only available when score window is not displayed
 	}
 	// Instant quit with CANCEL at the very beginning
-	if (nav == input::NAV_CANCEL && time < 1.0) {
-		Game::getSingletonPtr()->activateScreen("Playlist");
+	if (nav == input::NavButton::CANCEL && time < 1.0) {
+		if (m_menu.isOpen()) { m_menu.moveToLast(); }
+		else { Game::getSingletonPtr()->activateScreen(config["game/autoplay"].b() ? "Songs" : "Playlist"); }
 		return;
 	}
 
@@ -270,48 +299,56 @@ void ScreenSing::manageEvent(input::NavEvent const& event) {
 		input::DevicePtr dev = gm->controllers.registerDevice(event.source);
 		if (dev) {
 			// Eat all events and see if any are valid for joining
-			input::DevType type = input::DEVTYPE_GENERIC;
+			input::DevType type = input::DevType::GENERIC;
 			std::string msg;
 			for (input::Event ev; dev->getEvent(ev);) {
 				if (ev.value == 0.0) continue;
-				if (dev->type == input::DEVTYPE_DANCEPAD && m_song->hasDance()) {
-					if (ev.button == input::DANCEPAD_UP) type = dev->type;
+				if (dev->type == input::DevType::DANCEPAD && m_song->hasDance()) {
+					if (ev.button == input::ButtonId::DANCEPAD_UP) type = dev->type;
 					else msg = dev->source.isKeyboard() ? _("Press UP to join dance!") : _("Step UP to join!");
 				}
-				else if (dev->type == input::DEVTYPE_GUITAR && m_song->hasGuitars()) {
-					if (ev.button == input::GUITAR_GREEN) type = dev->type;
-					else if (ev.button != input::GUITAR_WHAMMY && ev.button != input::GUITAR_GODMODE) {
+				else if (dev->type == input::DevType::GUITAR && m_song->hasGuitars()) {
+					if (ev.button == input::ButtonId::GUITAR_GREEN) type = dev->type;
+					else if (ev.button != input::ButtonId::GUITAR_WHAMMY && ev.button != input::ButtonId::GUITAR_GODMODE) {
 						msg = dev->source.isKeyboard() ? _("Press 1 to join guitar!") : _("Press GREEN to join!");
 					}
 				}
-				else if (dev->type == input::DEVTYPE_DRUMS && m_song->hasDrums()) {
-					if (ev.button == input::DRUMS_KICK) type = dev->type;
+				else if (dev->type == input::DevType::DRUMS && m_song->hasDrums()) {
+					if (ev.button == input::ButtonId::DRUMS_KICK) type = dev->type;
 					else msg = dev->source.isKeyboard() ? _("Press SPACE to join drums!") : _("KICK to join!");
 				}
 			}
 			if (!msg.empty()) gm->flashMessage(msg, 0.0, 0.1, 0.1);
-			else if (type == input::DEVTYPE_DANCEPAD) m_instruments.push_back(new DanceGraph(m_audio, *m_song, dev));
-			else if (type != input::DEVTYPE_GENERIC) m_instruments.push_back(new GuitarGraph(m_audio, *m_song, dev, m_instruments.size()));
+			else if (type == input::DevType::DANCEPAD) m_instruments.push_back(std::make_unique<DanceGraph>(m_audio, *m_song, dev));
+			else if (type != input::DevType::GENERIC) m_instruments.push_back(std::make_unique<GuitarGraph>(m_audio, *m_song, dev, m_instruments.size()));
 		}
 	}
 
 	// Only pause or esc opens the global menu (instruments have their own menus)
 	// TODO: This should probably check if the source is participating as an instrument or not rather than check for its type
-	if (!devCanParticipate(event.devType) && (nav == input::NAV_PAUSE || nav == input::NAV_CANCEL) && !m_audio.isPaused() && !m_menu.isOpen()) {
+	if (!devCanParticipate(event.devType) && (nav == input::NavButton::PAUSE || nav == input::NavButton::CANCEL) && !m_audio.isPaused() && !m_menu.isOpen()) {
 		m_menu.open();
 		m_audio.togglePause();
 	}
 	// Global/singer pause menu navigation
 	if (m_menu.isOpen()) {
 		int do_action = 0;
-		if (nav == input::NAV_START) { do_action = 1; }
-		else if (nav == input::NAV_LEFT) { do_action = -1; }
-		else if (nav == input::NAV_RIGHT) { do_action = 1; }
-		else if (nav == input::NAV_DOWN) { m_menu.move(1); return; }
-		else if (nav == input::NAV_UP) { m_menu.move(-1); return; }
+		if (nav == input::NavButton::START) { do_action = 1; }
+		else if (nav == input::NavButton::LEFT) {
+			if (m_menu.current().type == MenuOption::Type::CHANGE_VALUE) { do_action = -1; }
+			else { m_menu.move(-1); return; }
+		}
+		else if (nav == input::NavButton::RIGHT) {
+			if (m_menu.current().type == MenuOption::Type::CHANGE_VALUE) { do_action = 1; }
+			else { m_menu.move(1); return; }
+			}
+		else if (nav == input::NavButton::DOWN) { m_menu.move(1); return; }
+		else if (nav == input::NavButton::UP) { m_menu.move(-1); return; }
 
 		if (do_action != 0) {
+			std::string currentOption = m_menu.current().getVirtName();
 			m_menu.action(do_action);
+			if (currentOption == "song/duet") { prepareVoicesMenu(m_menu.curIndex()); }
 			// Did the action close the menu?
 			if (!m_menu.isOpen() && m_audio.isPaused()) {
 				m_audio.togglePause();
@@ -320,20 +357,20 @@ void ScreenSing::manageEvent(input::NavEvent const& event) {
 		}
 	}
 	// Start button has special functions for skipping things (only in singing for now)
-	if (nav == input::NAV_START && m_instruments.empty() && !m_layout_singer.empty() && !m_audio.isPaused()) {
+	if (nav == input::NavButton::START && m_instruments.empty() && !m_layout_singer.empty() && !m_audio.isPaused()) {
 		// Open score dialog early
-		if (status == Song::FINISHED) {
+		if (status == Song::Status::FINISHED) {
 			if (m_engine) m_engine->kill(); // Kill the engine thread
-			m_score_window.reset(new ScoreWindow(m_instruments, m_database)); // Song finished, but no score window -> show it
+			m_score_window = std::make_unique<ScoreWindow>(m_instruments, m_database); // Song finished, but no score window -> show it
 		}
 		// Skip instrumental breaks
-		else if (status == Song::INSTRUMENTAL_BREAK) {
+		else if (status == Song::Status::INSTRUMENTAL_BREAK) {
 			if (time < 0) m_audio.seek(0.0);
 			else {
 				// TODO: Instead of calculating here, calculate instrumental breaks right after song loading and store in Song data structures
 				double diff = getNaN();
 				for (size_t i = 0; i < m_layout_singer.size(); ++i) {
-					double d = m_layout_singer[i].lyrics_begin() - 3.0 - time;
+					double d = m_layout_singer[i]->lyrics_begin() - 3.0 - time;
 					if (!(d > diff)) diff = d;  // Store smallest d in diff (notice NaN handling)
 				}
 				if (diff > 0.0) m_audio.seek(diff);
@@ -348,7 +385,7 @@ void ScreenSing::manageEvent(SDL_Event event) {
 	double time = m_audio.getPosition();
 	SDL_Scancode key = event.key.keysym.scancode;
 	// Ctrl combinations that can be used while performing (not when score dialog is displayed)
-	if (event.type == SDL_KEYDOWN && (event.key.keysym.mod & KMOD_LCTRL) && !m_score_window.get()) {
+	if (event.type == SDL_KEYDOWN && (event.key.keysym.mod & Platform::shortcutModifier(false)) && !m_score_window.get()) {
 		if (key == SDL_SCANCODE_C) {
 			m_audio.toggleCenterChannelSuppressor();
 			++config["audio/suppress_center_channel"];
@@ -369,7 +406,7 @@ void ScreenSing::manageEvent(SDL_Event event) {
 		// Toggle webcam
 		if (key == SDL_SCANCODE_A && Webcam::enabled()) {
 			// Initialize if we haven't done that already
-			if (!m_cam) { try { m_cam.reset(new Webcam(config["graphic/webcamid"].i())); } catch (...) { }; }
+			if (!m_cam) { try { m_cam = std::make_unique<Webcam>(config["graphic/webcamid"].i()); } catch (...) { }; }
 			if (m_cam) { dispInFlash(++config["graphic/webcam"]); m_cam->pause(!config["graphic/webcam"].b()); }
 		}
 		// Latency settings
@@ -405,7 +442,7 @@ void ScreenSing::manageEvent(SDL_Event event) {
 		// Some things must be reset after seeking backwards
 		if (seekback)
 			for (unsigned i = 0; i < m_layout_singer.size(); ++i)
-				m_layout_singer[i].reset();
+				m_layout_singer[i]->reset();
 		// Reload current song
 		if (key == SDL_SCANCODE_R) {
 			exit(); m_song->reload(); enter();
@@ -442,7 +479,7 @@ void ScreenSing::prepare() {
 	// except for joining, in which case global menu is closed
 	if (m_menu.isOpen()) {
 		for (auto& i: m_instruments) {
-			if (i.joining(time)) m_menu.close(); else i.toggleMenu(0);
+			if (i->joining(time)) m_menu.close(); else i->toggleMenu(0);
 		}
 	}
 }
@@ -450,9 +487,9 @@ void ScreenSing::prepare() {
 /// Test if a given device type can join the current song.
 // TODO: Somehow avoid duplicating these same checks in ScreenSing::prepare.
 bool ScreenSing::devCanParticipate(input::DevType const& devType) const {
-	if (devType == input::DEVTYPE_DANCEPAD && m_song->hasDance()) return true;
-	if (devType == input::DEVTYPE_GUITAR && m_song->hasGuitars()) return true;
-	if (devType == input::DEVTYPE_DRUMS && m_song->hasDrums()) return true;
+	if (devType == input::DevType::DANCEPAD && m_song->hasDance()) return true;
+	if (devType == input::DevType::GUITAR && m_song->hasGuitars()) return true;
+	if (devType == input::DevType::DRUMS && m_song->hasDrums()) return true;
 	return false;	
 }
 
@@ -469,7 +506,7 @@ void ScreenSing::draw() {
 		Transform ft(farTransform());
 		double ar = arMax;
 		// Background image
-		if (!m_background || m_background->empty()) m_background.reset(new Surface(m_backgrounds.getRandom()));
+		if (!m_background || m_background->empty()) m_background = std::make_unique<Texture>(m_backgrounds.getRandom());
 		ar = m_background->dimensions.ar();
 		if (ar > arMax || (m_video && ar > arMin)) fillBG();  // Fill white background to avoid black borders
 		m_background->draw();
@@ -488,16 +525,16 @@ void ScreenSing::draw() {
 		theme->bg_top.draw();
 	}
 
-	for (unsigned i = 0; i < m_layout_singer.size(); ++i) m_layout_singer[i].hideLyrics(m_audio.isPaused());
+	for (unsigned i = 0; i < m_layout_singer.size(); ++i) m_layout_singer[i]->hideLyrics(m_audio.isPaused());
 
 	instrumentLayout(time);
 
 	bool fullSinger = m_instruments.empty() && m_layout_singer.size() <= 1;
 	for (unsigned i = 0; i < m_layout_singer.size(); ++i) {
-		m_layout_singer[i].draw(time, fullSinger ? LayoutSinger::FULL : (i == 0 ? LayoutSinger::TOP : LayoutSinger::BOTTOM));
+		m_layout_singer[i]->draw(time, fullSinger ? LayoutSinger::PositionMode::FULL : (i == 0 ? LayoutSinger::PositionMode::TOP : LayoutSinger::PositionMode::BOTTOM));
 	}
 
-	Song::Status status = m_song->status(time);
+	Song::Status status = m_song->status(time, this);
 
 	// Compute and draw the timer and the progressbar
 	{
@@ -511,27 +548,38 @@ void ScreenSing::draw() {
 		std::string statustxt;
 		if (m_song->getPrevSection(t - 1.0, section)) {
 			statustxt = (boost::format("%02u:%02u - %s") % (t / 60) % (t % 60) % section.name).str();
-		} else  statustxt = (boost::format("%02u:%02u") % (t / 60) % (t % 60)).str();
+		} else {
+			statustxt = (boost::format("%02u:%02u") % (t / 60) % (t % 60)).str();
+		}
 
 		if (!m_score_window.get() && m_instruments.empty() && !m_layout_singer.empty()) {
-			if (status == Song::INSTRUMENTAL_BREAK)  statustxt += _("   ENTER to skip instrumental break");
-			if (status == Song::FINISHED && !config["game/karaoke_mode"].i()) {
+			if (status == Song::Status::INSTRUMENTAL_BREAK) {
+				statustxt += _("   ENTER to skip instrumental break");
+			}
+			if (status == Song::Status::FINISHED && !config["game/karaoke_mode"].i()) {
 				if(config["game/autoplay"].b()) {
 					if(m_displayAutoPlay) {
 						statustxt += _("   Autoplay enabled");
-					}
-					else {
-						 statustxt += _("   Remember to wait for grading!");
+					} else {
+						if(!m_audio.analyzers().empty()) {
+							statustxt += _("   Remember to wait for grading!");
+						} else {
+							statustxt += _("   Prepare for the next song!");
+						}
 					}
 
 					if(m_statusTextSwitch.get() == 0) {
-					m_displayAutoPlay = !m_displayAutoPlay;
-					m_statusTextSwitch.setValue(1);
+						m_displayAutoPlay = !m_displayAutoPlay;
+						m_statusTextSwitch.setValue(1);
 					}
+				} else {
+					if(!m_audio.analyzers().empty()) {
+						statustxt += _("   Remember to wait for grading!");
 					} else {
-					statustxt += _("   Remember to wait for grading!");
+						statustxt += _("   Choose your next song!");
+					}
 				}
-			} else if(status == Song::FINISHED && config["game/autoplay"].b()) {
+			} else if(status == Song::Status::FINISHED && config["game/autoplay"].b()) {
 				statustxt += _("   Autoplay enabled");
 			}
 		}
@@ -555,17 +603,17 @@ void ScreenSing::draw() {
 				m_score_window->draw();
 			}
 		}
-		else if (!m_audio.isPlaying() || (status == Song::FINISHED
+		else if (!m_audio.isPlaying() || (status == Song::Status::FINISHED
 		  && m_audio.getLength() - time <= (m_song->instrumentTracks.empty() && m_song->danceTracks.empty() ? 3.0 : 0.2) )) {
 			// Time to create the score window
-			m_quitTimer.setValue(QUIT_TIMEOUT);
+			m_quitTimer.setValue(config["game/results_timeout"].i());
 			if (m_engine) m_engine->kill(); // kill the engine thread (to avoid consuming memory)
-			m_score_window.reset(new ScoreWindow(m_instruments, m_database));
+			m_score_window = std::make_unique<ScoreWindow>(m_instruments, m_database);
 		}
 	}
 
 	// Menus on top of everything
-	for (auto& i: m_instruments) if (i.menuOpen()) i.drawMenu();
+	for (auto& i: m_instruments) if (i->menuOpen()) i->drawMenu();
 	if (m_menu.isOpen()) drawMenu();
 	if(!keyPressed && m_DuetTimeout.get() == 0) {
 		m_menu.action();
@@ -590,8 +638,8 @@ void ScreenSing::drawMenu() {
 	th.bg.draw();
 	// Loop through menu items
 	w = 0;
-	int player = 0;
-	boost::ptr_vector<Analyzer>& analyzers = m_audio.analyzers();
+	std::size_t player = 0;
+	auto& analyzers = m_audio.analyzers();
 	for (MenuOptions::const_iterator it = m_menu.begin(); it != m_menu.end(); ++it) {
 		// Pick the font object
 		SvgTxtTheme* txt = &th.option_selected;
@@ -601,17 +649,20 @@ void ScreenSing::drawMenu() {
 		txt->dimensions.middle(x).center(y);
 		txt->draw(it->getName());
 		if (it->value == &m_vocalTracks[player]) {
-			Color color = MicrophoneColor::get(analyzers[player].getId());
-			ColorTrans c(color);
-			m_player_icon->dimensions.right(x).fixedHeight(0.040).center(y);
-			m_player_icon->draw();
+			if (player < analyzers.size()) {
+				Color color = MicrophoneColor::get(analyzers[player].getId());
+				ColorTrans c(color);
+				m_player_icon->dimensions.right(x).fixedHeight(0.040).center(y);
+				m_player_icon->draw();
+			}
 			player++;
 		}
+
 		w = std::max(w, txt->w() + 2 * step); // Calculate the widest entry
 		y += step;
 	}
 	if (cur->getComment() != "") {
-		th.comment.dimensions.middle(0).screenBottom(-0.12);
+		th.comment.dimensions.middle(0).screenBottom(-0.08);
 		th.comment.draw(cur->getComment());
 	}
 	m_menu.dimensions.stretch(w, h);
@@ -622,7 +673,7 @@ ScoreWindow::ScoreWindow(Instruments& instruments, Database& database):
   m_database(database),
   m_pos(0.8, 2.0),
   m_bg(findFile("score_window.svg")),
-  m_scoreBar(findFile("score_bar_bg.svg"), findFile("score_bar_fg.svg"), ProgressBar::VERTICAL, 0.0, 0.0, false),
+  m_scoreBar(findFile("score_bar_bg.svg"), findFile("score_bar_fg.svg"), ProgressBar::Mode::VERTICAL, 0.0, 0.0, false),
   m_score_text(findFile("score_txt.svg")),
   m_score_rank(findFile("score_rank.svg"))
 {
@@ -631,7 +682,7 @@ ScoreWindow::ScoreWindow(Instruments& instruments, Database& database):
 	m_database.scores.clear();
 	// Singers
 	for (auto p = m_database.cur.begin(); p != m_database.cur.end();) {
-		ScoreItem item; item.type = input::DEVTYPE_VOCALS;
+		ScoreItem item; item.type = input::DevType::VOCALS;
 		item.score = p->getScore();
 		if (item.score < 500) { p = m_database.cur.erase(p); continue; } // Dead
 		item.track = "Vocals"; // For database
@@ -644,12 +695,12 @@ ScoreWindow::ScoreWindow(Instruments& instruments, Database& database):
 	// Instruments
 	for (Instruments::iterator it = instruments.begin(); it != instruments.end();) {
 		ScoreItem item;
-		item.type = it->getGraphType();
-		item.score = it->getScore();
+		item.type = (*it)->getGraphType();
+		item.score = (*it)->getScore();
 		if (item.score < 100) { it = instruments.erase(it); continue; } // Dead
-		item.track_simple = it->getTrack();
-		item.track = it->getModeId();
-		item.track[0] = toupper(item.track[0]); // Capitalize
+		item.track_simple = (*it)->getTrack();
+		item.track = (*it)->getModeId();
+		item.track = UnicodeUtil::toUpper(item.track, 1); // Capitalize
 		if (item.track_simple == TrackName::DRUMS) item.color = Color(0.1, 0.1, 0.1);
 		else if (item.track_simple == TrackName::BASS) item.color = Color(0.5, 0.3, 0.1);
 		else item.color = Color(1.0, 0.0, 0.0);
@@ -662,16 +713,17 @@ ScoreWindow::ScoreWindow(Instruments& instruments, Database& database):
 		m_rank = _("No player!");
 	else {
 		// Determine winner
-		ScoreItem winner = *std::min_element(m_database.scores.begin(), m_database.scores.end());  // Note: best score comes first
+		m_database.scores.sort([](ScoreItem i, ScoreItem j) -> bool { return (i.score>j.score); });
+		ScoreItem winner = *std::max_element(m_database.scores.begin(), m_database.scores.end());
 		int topScore = winner.score;
 		// Determine rank
-		if (winner.type == input::DEVTYPE_VOCALS) {
+		if (winner.type == input::DevType::VOCALS) {
 			if (topScore > 8000) m_rank = _("Hit singer");
 			else if (topScore > 6000) m_rank = _("Lead singer");
 			else if (topScore > 4000) m_rank = _("Rising star");
 			else if (topScore > 2000) m_rank = _("Amateur");
 			else m_rank = _("Tone deaf");
-		} else if (winner.type == input::DEVTYPE_DANCEPAD) {
+		} else if (winner.type == input::DevType::DANCEPAD) {
 			if (topScore > 8000) m_rank = _("Maniac");
 			else if (topScore > 6000) m_rank = _("Hoofer");
 			else if (topScore > 4000) m_rank = _("Rising star");
@@ -701,7 +753,7 @@ void ScoreWindow::draw() {
 		double x = spacing * (0.5 + i - 0.5 * m_database.scores.size());
 		m_scoreBar.dimensions.fixedWidth(0.09).middle(x).bottom(0.20);
 		m_scoreBar.draw(score / 10000.0);
-		m_score_text.render(boost::lexical_cast<std::string>(score));
+		m_score_text.render(std::to_string(score));
 		m_score_text.dimensions().middle(x).top(0.24).fixedHeight(0.042);
 		m_score_text.draw();
 		m_score_text.render(p->track_simple);
