@@ -5,20 +5,23 @@
 #include "database.hh"
 #include "hiscore.hh"
 #include "i18n.hh"
+#include "platform.hh"
 #include "screen_sing.hh"
 #include "screen_playlist.hh"
 #include "songs.hh"
 #include "theme.hh"
 #include "util.hh"
 #include "playlist.hh"
-#include <iostream>
-#include <sstream>
-#include <boost/format.hpp>
 
-static const double IDLE_TIMEOUT = 45.0; // seconds
+#include "aubio/aubio.h"
+#include <iostream>
+#include <mutex>
+#include <sstream>
+
+static const double IDLE_TIMEOUT = 35.0; // seconds
 
 ScreenSongs::ScreenSongs(std::string const& name, Audio& audio, Songs& songs, Database& database):
-  Screen(name), m_audio(audio), m_songs(songs), m_database(database), m_covers(50)
+  Screen(name), m_audio(audio), m_songs(songs), m_database(database)
 {
 	m_songs.setAnimMargins(5.0, 5.0);
 	// Using AnimValues as a simple timers counting seconds
@@ -37,15 +40,15 @@ void ScreenSongs::enter() {
 }
 
 void ScreenSongs::reloadGL() {
-	theme.reset(new ThemeSongs());
-	m_menuTheme.reset(new ThemeInstrumentMenu());
-	m_songbg_default.reset(new Surface(findFile("songs_bg_default.svg")));
-	m_songbg_ground.reset(new Surface(findFile("songs_bg_ground.svg")));
-	m_singCover.reset(new Surface(findFile("no_cover.svg")));
-	m_instrumentCover.reset(new Surface(findFile("instrument_cover.svg")));
-	m_bandCover.reset(new Surface(findFile("band_cover.svg")));
-	m_danceCover.reset(new Surface(findFile("dance_cover.svg")));
-	m_instrumentList.reset(new Texture(findFile("instruments.svg")));
+	theme = std::make_unique<ThemeSongs>();
+	m_menuTheme = std::make_unique<ThemeInstrumentMenu>();
+	m_songbg_default = std::make_unique<Texture>(findFile("songs_bg_default.svg"));
+	m_songbg_ground = std::make_unique<Texture>(findFile("songs_bg_ground.svg"));
+	m_singCover = std::make_unique<Texture>(findFile("no_cover.svg"));
+	m_instrumentCover = std::make_unique<Texture>(findFile("instrument_cover.svg"));
+	m_bandCover = std::make_unique<Texture>(findFile("band_cover.svg"));
+	m_danceCover = std::make_unique<Texture>(findFile("dance_cover.svg"));
+	m_instrumentList = std::make_unique<Texture>(findFile("instruments.svg"));
 }
 
 void ScreenSongs::exit() {
@@ -81,25 +84,25 @@ void ScreenSongs::manageEvent(input::NavEvent const& event) {
 	input::NavButton nav = event.button;
 	// Handle basic navigational input that is possible also with instruments
 	m_idleTimer.setValue(0.0);  // Reset idle timer
-	if (nav == input::NAV_PAUSE) m_audio.togglePause();
-	else if (event.menu == input::NAVMENU_A_PREV) {
+	if (nav == input::NavButton::PAUSE) m_audio.togglePause();
+	else if (event.menu == input::NavMenu::A_PREV) {
 		if (m_menu.isOpen()) m_menu.move(-1);
 		else menuBrowse(-1);
 	}
-	else if (event.menu == input::NAVMENU_A_NEXT) {
+	else if (event.menu == input::NavMenu::A_NEXT) {
 		if (m_menu.isOpen()) m_menu.move(1);
 		else menuBrowse(1);
 	}
-	else if (nav == input::NAV_MOREUP) m_songs.advance(-10);
-	else if (nav == input::NAV_MOREDOWN) m_songs.advance(10);
+	else if (nav == input::NavButton::MOREUP) m_songs.advance(-10);
+	else if (nav == input::NavButton::MOREDOWN) m_songs.advance(10);
 	else if (m_jukebox) {
-		if (nav == input::NAV_CANCEL) m_jukebox = false;
-		else if (nav == input::NAV_START) { addSong(); sing(); }
-		else if (event.menu == input::NAVMENU_B_NEXT)  m_audio.seek(-5);
-		else if (event.menu == input::NAVMENU_B_PREV) m_audio.seek(5);
-		else if (nav == input::NAV_MOREUP) m_audio.seek(-30);
-		else if (nav == input::NAV_MOREDOWN) m_audio.seek(30);
-	} else if (nav == input::NAV_CANCEL) {
+		if (nav == input::NavButton::CANCEL) m_jukebox = false;
+		else if (nav == input::NavButton::START) { addSong(); sing(); }
+		else if (event.menu == input::NavMenu::B_NEXT)  m_audio.seek(-5);
+		else if (event.menu == input::NavMenu::B_PREV) m_audio.seek(5);
+		else if (nav == input::NavButton::MOREUP) m_audio.seek(-30);
+		else if (nav == input::NavButton::MOREDOWN) m_audio.seek(30);
+	} else if (nav == input::NavButton::CANCEL) {
 		if (m_menuPos != 1) m_menuPos = 1;  // Exit menu (back to song selection)
 		else if (!m_search.text.empty()) { m_search.text.clear(); m_songs.setFilter(m_search.text); }  // Clear search
 		else if (m_songs.typeNum()) m_songs.typeChange(0);  // Clear type filter
@@ -107,7 +110,7 @@ void ScreenSongs::manageEvent(input::NavEvent const& event) {
 	}
 	// The rest are only available when there are songs available
 	else if (m_songs.empty()) return;
-	else if (nav == input::NAV_START) {
+	else if (nav == input::NavButton::START) {
 		if (m_menu.isOpen()) {
 			m_menu.action();
 		}
@@ -128,11 +131,11 @@ void ScreenSongs::manageEvent(input::NavEvent const& event) {
 			}
 		}
 	}
-	else if (event.menu == input::NAVMENU_B_PREV) {
+	else if (event.menu == input::NavMenu::B_PREV) {
 		if (m_menu.isOpen()) m_menu.move(-1);
 		else if (m_menuPos < 4) ++m_menuPos;
 	}
-	else if (event.menu == input::NAVMENU_B_NEXT) {
+	else if (event.menu == input::NavMenu::B_NEXT) {
 		if (m_menu.isOpen()) m_menu.move(1);
 		else if (m_menuPos > 0) --m_menuPos;
 	}
@@ -154,7 +157,7 @@ void ScreenSongs::manageEvent(SDL_Event event) {
 			m_songs.setFilter(m_search.text);
 		}
 		else if (!m_jukebox) {
-			if (key == SDL_SCANCODE_R && mod & KMOD_CTRL) {
+			if (key == SDL_SCANCODE_R && mod & Platform::shortcutModifier()) {
 				m_songs.reload();
 				m_songs.setFilter(m_search.text);
 				}
@@ -188,13 +191,18 @@ void ScreenSongs::update() {
 		}
 	}
 	// Check out if the music has changed
-	boost::shared_ptr<Song> song = m_songs.currentPtr();
-	Song::Music music;
+	std::shared_ptr<Song> song = m_songs.currentPtr();
+	Song::MusicFiles music;
 	if (song) music = song->music;
 	if (m_playing != music) songChange = true;
 	// Switch songs if needed, only when the user is not browsing for a moment
 	if (!songChange) return;
-	if (song) song->loadNotes();  // Needed for BPM info; TODO: drop notes when switching to another song.
+	ScreenSongs::previewBeatsBuffer.reset(new_fvec(1));
+	{
+	std::lock_guard<std::recursive_mutex> l(Audio::aubio_mutex);	
+	Audio::aubioTempo.reset(new_aubio_tempo("default", Audio::aubio_win_size, Audio::aubio_hop_size, Audio::getSR()));
+	}
+	if (song && song->hasControllers()) { song->loadNotes(); } // Needed for BPM info.
 	m_playing = music;
 	// Clear the old content and load new content if available
 	m_songbg.reset(); m_video.reset();
@@ -202,8 +210,8 @@ void ScreenSongs::update() {
 	m_audio.playMusic(music, true, 1.0, pstart);
 	if (song) {
 		fs::path const& background = song->background.empty() ? song->cover : song->background;
-		if (!background.empty()) try { m_songbg.reset(new Surface(background)); } catch (std::exception const&) {}
-		if (!song->video.empty() && config["graphic/video"].b()) m_video.reset(new Video(song->video, song->videoGap));
+		if (!background.empty()) try { m_songbg = std::make_unique<Texture>(background); } catch (std::exception const&) {}
+		if (!song->video.empty() && config["graphic/video"].b()) m_video = std::make_unique<Video>(song->video, song->videoGap);
 	}
 }
 
@@ -236,10 +244,10 @@ void ScreenSongs::drawJukebox() {
 	if (diff < 3.0) {
 		Song& song = m_songs.current();
 		// Draw the cover
-		Surface* cover = NULL;
-		if (!song.cover.empty()) try { cover = &m_covers[song.cover]; } catch (std::exception const&) {}
+		Texture* cover = nullptr;
+		if (!song.cover.empty()) cover = loadTextureFromMap(song.cover);
 		if (cover && !cover->empty()) {
-			Surface& s = *cover;
+			Texture& s = *cover;
 			s.dimensions.left(theme->song.dimensions.x1()).top(theme->song.dimensions.y2() + 0.05).fitInside(0.15, 0.15);
 			s.draw();
 		}
@@ -260,7 +268,7 @@ void ScreenSongs::drawMultimedia() {
 			if (m_songbg->width() > 512 && m_songbg->dimensions.ar() > 1.1) {
 				// Full screen mode
 				float s = sin(m_clock.get()) * 0.15 + 1.15;
-				Transform sc(scale(glmath::vec3(s, s, s)));
+				Transform sc(glmath::scale(glmath::vec3(s, s, s)));
 				m_songbg->draw();
 			} else {
 				// Low res texture or cover image, render in tiled mode
@@ -344,30 +352,48 @@ void ScreenSongs::drawCovers() {
 	double beat = 0.5 + m_idleTimer.get() / 2.0;  // 30 BPM
 	if (ss > 0) {
 		// Use actual song BPM. FIXME: Should only do this if currentId is also playing.
-		double t = m_audio.getPosition() - config["audio/video_delay"].f();
-		Song::Beats const& beats = m_songs.current().beats;
-		auto it = std::lower_bound(beats.begin(), beats.end(), t);
-		if (it != beats.begin() && it != beats.end()) {
-			double t1 = *(it - 1), t2 = *it;
-			beat = (t - t1) / (t2 - t1);
+		if (m_songs.currentPtr()->music == m_playing) {
+				if (m_songs.currentPtr()->hasControllers() || !m_songs.currentPtr()->beats.empty()) {
+				double t = m_audio.getPosition() - config["audio/video_delay"].f();
+				Song::Beats const& beats = m_songs.current().beats;
+				auto it = std::lower_bound(m_songs.currentPtr()->hasControllers() ? beats.begin() : (beats.begin() + 1), beats.end(), t);
+				if (it != beats.begin() && it != beats.end()) {
+					double t1 = *(it - 1), t2 = *it;
+					beat = (t - t1) / (t2 - t1);
+				}
+			}
+			else if (m_songs.currentPtr() && !m_songs.currentPtr()->m_bpms.empty()) {
+				double tempo = (m_songs.currentPtr()->m_bpms.front().step * 4.0);
+				if (int(tempo) <= 100.0) tempo *= 2.0;
+				else if (int(tempo) > 400.0) tempo /= 4.0;
+				else if (int(tempo) > 300.0) tempo /= 3.0;
+				else if (int(tempo) > 190.0) tempo /= 2.0;
+				beat = 0.5 + m_idleTimer.get() / tempo;
+			}
 		}
 	}
-	beat = 1.0 + std::pow(std::abs(std::cos(3.141592 * beat)), 10.0);  // Overdrive pulse
+	beat = 1.0 + std::pow(std::abs(std::cos(0.5 * TAU * beat)), 10.0);  // Overdrive pulse
 	// Draw covers and reflections
 	for (int i = -2; i < 6; ++i) {
 		if (baseidx + i < 0 || baseidx + i >= int(ss)) continue;
 		Song& song = *m_songs[baseidx + i];
-		Surface& s = getCover(song);
+		Texture& s = getCover(song);
 		// Calculate dimensions for cover and instrument markers
-		double diff = 0.5 * (1.0 + std::cos(std::min(M_PI, std::abs(i - shift))));  // 0..1 for current cover hilight level
+		double pos = i - shift;
+		// Function for highlight effect (offset = 0 for current cover), returns 0..1 highlight level
+		auto highlightf = [=](double offset) { return smoothstep(3.5, 0.0, std::abs(pos + offset)); };
+		// Coordinate translations (pos and offset in cover units to z and x in OpenGL space)
+		auto ztrans = [=](double offset) { return -0.5 + 0.3 * highlightf(offset); };
+		auto xtrans = [=](double offset) { return -0.2 + 0.20 * (pos + offset); };
+		// A cover is angled to a line between the surrounding gaps (offset +- 0.5 covers)
+		double angle = -std::atan2(ztrans(0.5) - ztrans(-0.5), xtrans(0.5) - xtrans(-0.5));
 		double y = 0.5 * virtH();
-		using namespace glmath;
-		Transform trans(
-		  translate(vec3(-0.2 + 0.20 * (i - shift), y, -0.2 - 0.3 * (1.0 - diff)))
-		  * rotate(0.4 * std::sin(std::min(M_PI, i - shift)), vec3(0.0, 1.0, 0.0))
-		);
-		double c = 0.4 + 0.6 * diff;
+		double x = xtrans(0.0);
+		double z = ztrans(0.0);
+		double c = 0.4 + 0.6 * highlightf(0.0);
 		if (m_menuPos == 1 /* Cover browser */ && baseidx + i == currentId) c = beat;
+		using namespace glmath;
+		Transform trans(translate(vec3(x, y, z)) * rotate(angle, vec3(0.0, 1.0, 0.0)));
 		ColorTrans c1(Color(c, c, c));
 		s.dimensions.middle().screenCenter().bottom().fitInside(0.17, 0.17);
 		// Draw the cover normally
@@ -378,12 +404,12 @@ void ScreenSongs::drawCovers() {
 		s.draw();
 	}
 	// Draw the playlist
-    Game* gm = Game::getSingletonPtr();
+	Game* gm = Game::getSingletonPtr();
 	auto const& playlist = gm->getCurrentPlayList().getList();
 	double c = (m_menuPos == 0 /* Playlist */ ? beat : 1.0);
 	ColorTrans c1(Color(c, c, c));
 	for (unsigned i = playlist.size() - 1; i < playlist.size(); --i) {
-		Surface& s = getCover(*playlist[i]);
+		Texture& s = getCover(*playlist[i]);
 		double pos =  i / std::max<double>(5.0, playlist.size());
 		using namespace glmath;
 		Transform trans(
@@ -395,12 +421,23 @@ void ScreenSongs::drawCovers() {
 	}
 }
 
-Surface& ScreenSongs::getCover(Song const& song) {
-	Surface* cover = nullptr;
+Texture* ScreenSongs::loadTextureFromMap(fs::path path) {
+	if(m_covers.find(path) == m_covers.end()) {
+		std::pair<fs::path, std::unique_ptr<Texture>> kv = std::make_pair(path, std::make_unique<Texture>(path));
+		m_covers.insert(std::move(kv));
+	}
+	try {
+		return m_covers.at(path).get();
+	} catch (std::exception const&) {}
+	return nullptr;
+}
+
+Texture& ScreenSongs::getCover(Song const& song) {
+	Texture* cover = nullptr;
 	// Fetch cover image from cache or try loading it
-	if (!song.cover.empty()) try { cover = &m_covers[song.cover]; } catch (std::exception const&) {}
+	if (!song.cover.empty()) cover = loadTextureFromMap(song.cover);
 	// Fallback to background image as cover if needed
-	if (!cover && !song.background.empty()) try { cover = &m_covers[song.background]; } catch (std::exception const&) {}
+	if (!cover && !song.background.empty()) cover = loadTextureFromMap(song.background);
 	// Use empty cover
 	if (!cover) {
 		if(song.hasDance()) {
@@ -444,7 +481,7 @@ void ScreenSongs::drawInstruments(Dimensions dim) const {
 		have_drums = song.hasDrums();
 		have_dance = song.hasDance();
 		//is_karaoke = (song.music.find("vocals") != song.music.end());
-		vocalCount = song.getVocalTrackNames().size();
+		if (song.hasVocals()) vocalCount = song.hasDuet() ? 2 : 1; // Make sure our generated duet track is not counted as a third vocal track for drawing.
 		if (isTrackInside(song.instrumentTracks,TrackName::GUITAR)) guitarCount++;
 		if (isTrackInside(song.instrumentTracks,TrackName::GUITAR_COOP)) guitarCount++;
 		if (isTrackInside(song.instrumentTracks,TrackName::GUITAR_RHYTHM)) guitarCount++;
@@ -508,6 +545,8 @@ void ScreenSongs::drawMenu() {
 	m_menu.dimensions.stretch(w, h);
 }
 
+std::unique_ptr<fvec_t, void(*)(fvec_t*)> ScreenSongs::previewBeatsBuffer = std::unique_ptr<fvec_t, void(*)(fvec_t*)>(new_fvec(1), [](fvec_t* p){del_fvec(p);});
+
 void ScreenSongs::createPlaylistMenu() {
 	m_menu.clear();
 	m_menu.add(MenuOption(_("Play"), "").call([this]() {
